@@ -46,11 +46,23 @@ struct fat {
 
 struct alignas(64u) over_aligned {};
 
+TEST(Any, Empty) {
+    entt::any any{};
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
+    ASSERT_EQ(any.type(), entt::type_id<void>());
+    ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
+    ASSERT_EQ(any.data(), nullptr);
+}
+
 TEST(Any, SBO) {
     entt::any any{'c'};
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
     ASSERT_EQ(any.type(), entt::type_id<char>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<char>(any), 'c');
@@ -61,27 +73,21 @@ TEST(Any, NoSBO) {
     entt::any any{instance};
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(any.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(any), instance);
 }
 
-TEST(Any, Empty) {
-    entt::any any{};
-
-    ASSERT_FALSE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
-    ASSERT_EQ(any.type(), entt::type_id<void>());
-    ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
-    ASSERT_EQ(any.data(), nullptr);
-}
-
-TEST(Any, SBOInPlaceTypeConstruction) {
-    entt::any any{std::in_place_type<int>, 2};
+TEST(Any, SBOInPlaceConstruction) {
+    std::unique_ptr<int> elem = std::make_unique<int>(2);
+    // NOLINTNEXTLINE(clang-analyzer-cplusplus.NewDeleteLeaks)
+    entt::any any{std::in_place, elem.release()};
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(any.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<int>(any), 2);
@@ -89,6 +95,37 @@ TEST(Any, SBOInPlaceTypeConstruction) {
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
+    ASSERT_FALSE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::ref);
+    ASSERT_EQ(other.type(), entt::type_id<int>());
+    ASSERT_EQ(entt::any_cast<int>(other), 2);
+    ASSERT_EQ(other.data(), any.data());
+}
+
+TEST(Any, SBOInPlaceNullptrConstruction) {
+    int *instance = nullptr;
+    const entt::any any{std::in_place, instance};
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
+    ASSERT_EQ(any.type(), entt::type_id<void>());
+}
+
+TEST(Any, SBOInPlaceTypeConstruction) {
+    entt::any any{std::in_place_type<int>, 2};
+
+    ASSERT_TRUE(any);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
+    ASSERT_EQ(any.type(), entt::type_id<int>());
+    ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
+    ASSERT_EQ(entt::any_cast<int>(any), 2);
+
+    auto other = any.as_ref();
+
+    ASSERT_TRUE(other);
+    ASSERT_FALSE(other.owner());
     ASSERT_EQ(other.policy(), entt::any_policy::ref);
     ASSERT_EQ(other.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<int>(other), 2);
@@ -100,6 +137,7 @@ TEST(Any, SBOAsRefConstruction) {
     entt::any any{entt::forward_as_any(value)};
 
     ASSERT_TRUE(any);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::ref);
     ASSERT_EQ(any.type(), entt::type_id<int>());
 
@@ -118,6 +156,7 @@ TEST(Any, SBOAsRefConstruction) {
     any.emplace<int &>(value);
 
     ASSERT_TRUE(any);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::ref);
     ASSERT_EQ(any.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<int>(&any), &value);
@@ -125,6 +164,7 @@ TEST(Any, SBOAsRefConstruction) {
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
+    ASSERT_FALSE(other.owner());
     ASSERT_EQ(other.policy(), entt::any_policy::ref);
     ASSERT_EQ(other.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<int>(other), 2);
@@ -136,6 +176,7 @@ TEST(Any, SBOAsConstRefConstruction) {
     entt::any any{entt::forward_as_any(value)};
 
     ASSERT_TRUE(any);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::cref);
     ASSERT_EQ(any.type(), entt::type_id<int>());
 
@@ -154,6 +195,7 @@ TEST(Any, SBOAsConstRefConstruction) {
     any.emplace<const int &>(value);
 
     ASSERT_TRUE(any);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::cref);
     ASSERT_EQ(any.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<const int>(&any), &value);
@@ -161,6 +203,7 @@ TEST(Any, SBOAsConstRefConstruction) {
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
+    ASSERT_FALSE(other.owner());
     ASSERT_EQ(other.policy(), entt::any_policy::cref);
     ASSERT_EQ(other.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<int>(other), 2);
@@ -173,7 +216,8 @@ TEST(Any, SBOCopyConstruction) {
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::embedded);
     ASSERT_EQ(other.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
     ASSERT_EQ(entt::any_cast<int>(other), 2);
@@ -187,7 +231,8 @@ TEST(Any, SBOCopyAssignment) {
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::embedded);
     ASSERT_EQ(other.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
     ASSERT_EQ(entt::any_cast<int>(other), 2);
@@ -200,7 +245,8 @@ TEST(Any, SBOSelfCopyAssignment) {
     any = *&any;
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
     ASSERT_EQ(any.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<int>(any), 2);
@@ -214,7 +260,8 @@ TEST(Any, SBOMoveConstruction) {
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::embedded);
     ASSERT_EQ(other.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
     ASSERT_EQ(entt::any_cast<int>(other), 2);
@@ -229,17 +276,24 @@ TEST(Any, SBOMoveAssignment) {
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::embedded);
     ASSERT_EQ(other.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
     ASSERT_EQ(entt::any_cast<int>(other), 2);
 }
 
-ENTT_DEBUG_TEST(AnyDeathTest, SBOSelfMoveAssignment) {
+TEST(AnyDeathTest, SBOSelfMoveAssignment) {
     entt::any any{2};
 
     // avoid warnings due to self-assignment
-    ASSERT_DEATH(any = std::move(*&any), "");
+    any = std::move(*&any);
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
+    ASSERT_EQ(any.type(), entt::type_id<void>());
+    ASSERT_EQ(any.data(), nullptr);
 }
 
 TEST(Any, SBODirectAssignment) {
@@ -247,7 +301,8 @@ TEST(Any, SBODirectAssignment) {
     any = 2;
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
     ASSERT_EQ(any.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<int>(any), 2);
@@ -344,12 +399,45 @@ TEST(Any, SBOAsConstRefTransferValue) {
     ASSERT_EQ(value, 2);
 }
 
+TEST(Any, NoSBOInPlaceConstruction) {
+    std::unique_ptr<fat> elem = std::make_unique<fat>(.1, .2, .3, .4);
+    // NOLINTNEXTLINE(clang-analyzer-cplusplus.NewDeleteLeaks)
+    entt::any any{std::in_place, elem.release()};
+
+    ASSERT_TRUE(any);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
+    ASSERT_EQ(any.type(), entt::type_id<fat>());
+    ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
+    ASSERT_EQ(entt::any_cast<fat>(any), (fat{.1, .2, .3, .4}));
+
+    auto other = any.as_ref();
+
+    ASSERT_TRUE(other);
+    ASSERT_FALSE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::ref);
+    ASSERT_EQ(other.type(), entt::type_id<fat>());
+    ASSERT_EQ(entt::any_cast<fat>(other), (fat{.1, .2, .3, .4}));
+    ASSERT_EQ(other.data(), any.data());
+}
+
+TEST(Any, NoSBOInPlaceNullptrConstruction) {
+    fat *instance = nullptr;
+    const entt::any any{std::in_place, instance};
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
+    ASSERT_EQ(any.type(), entt::type_id<void>());
+}
+
 TEST(Any, NoSBOInPlaceTypeConstruction) {
     const fat instance{.1, .2, .3, .4};
     entt::any any{std::in_place_type<fat>, instance};
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(any.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(any), instance);
@@ -357,6 +445,7 @@ TEST(Any, NoSBOInPlaceTypeConstruction) {
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
+    ASSERT_FALSE(other.owner());
     ASSERT_EQ(other.policy(), entt::any_policy::ref);
     ASSERT_EQ(other.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<fat>(other), (fat{.1, .2, .3, .4}));
@@ -368,7 +457,7 @@ TEST(Any, NoSBOAsRefConstruction) {
     entt::any any{entt::forward_as_any(instance)};
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::ref);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::ref);
     ASSERT_EQ(any.type(), entt::type_id<fat>());
 
@@ -387,6 +476,7 @@ TEST(Any, NoSBOAsRefConstruction) {
     any.emplace<fat &>(instance);
 
     ASSERT_TRUE(any);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::ref);
     ASSERT_EQ(any.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<fat>(&any), &instance);
@@ -394,6 +484,7 @@ TEST(Any, NoSBOAsRefConstruction) {
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
+    ASSERT_FALSE(other.owner());
     ASSERT_EQ(other.policy(), entt::any_policy::ref);
     ASSERT_EQ(other.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<fat>(other), (fat{.1, .2, .3, .4}));
@@ -405,6 +496,7 @@ TEST(Any, NoSBOAsConstRefConstruction) {
     entt::any any{entt::forward_as_any(instance)};
 
     ASSERT_TRUE(any);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::cref);
     ASSERT_EQ(any.type(), entt::type_id<fat>());
 
@@ -423,6 +515,7 @@ TEST(Any, NoSBOAsConstRefConstruction) {
     any.emplace<const fat &>(instance);
 
     ASSERT_TRUE(any);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::cref);
     ASSERT_EQ(any.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<const fat>(&any), &instance);
@@ -430,6 +523,7 @@ TEST(Any, NoSBOAsConstRefConstruction) {
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
+    ASSERT_FALSE(other.owner());
     ASSERT_EQ(other.policy(), entt::any_policy::cref);
     ASSERT_EQ(other.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<fat>(other), (fat{.1, .2, .3, .4}));
@@ -443,7 +537,8 @@ TEST(Any, NoSBOCopyConstruction) {
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(other.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(other), instance);
@@ -458,7 +553,8 @@ TEST(Any, NoSBOCopyAssignment) {
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(other.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(other), instance);
@@ -472,7 +568,8 @@ TEST(Any, NoSBOSelfCopyAssignment) {
     any = *&any;
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(any.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(any), instance);
@@ -487,7 +584,8 @@ TEST(Any, NoSBOMoveConstruction) {
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(other.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(other), instance);
@@ -503,18 +601,25 @@ TEST(Any, NoSBOMoveAssignment) {
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(other.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(other), instance);
 }
 
-ENTT_DEBUG_TEST(AnyDeathTest, NoSBOSelfMoveAssignment) {
+TEST(AnyDeathTest, NoSBOSelfMoveAssignment) {
     const fat instance{.1, .2, .3, .4};
     entt::any any{instance};
 
     // avoid warnings due to self-assignment
-    ASSERT_DEATH(any = std::move(*&any), "");
+    any = std::move(*&any);
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
+    ASSERT_EQ(any.type(), entt::type_id<void>());
+    ASSERT_EQ(any.data(), nullptr);
 }
 
 TEST(Any, NoSBODirectAssignment) {
@@ -523,7 +628,8 @@ TEST(Any, NoSBODirectAssignment) {
     any = instance;
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(any.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(any), instance);
@@ -639,7 +745,8 @@ TEST(Any, VoidInPlaceTypeConstruction) {
     entt::any any{std::in_place_type<void>};
 
     ASSERT_FALSE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
     ASSERT_EQ(any.type(), entt::type_id<void>());
     ASSERT_EQ(entt::any_cast<int>(&any), nullptr);
 }
@@ -650,7 +757,8 @@ TEST(Any, VoidCopyConstruction) {
 
     ASSERT_FALSE(any);
     ASSERT_FALSE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::empty);
     ASSERT_EQ(other.type(), entt::type_id<void>());
     ASSERT_EQ(entt::any_cast<int>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
@@ -664,7 +772,8 @@ TEST(Any, VoidCopyAssignment) {
 
     ASSERT_FALSE(any);
     ASSERT_FALSE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::empty);
     ASSERT_EQ(other.type(), entt::type_id<void>());
     ASSERT_EQ(entt::any_cast<int>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
@@ -677,7 +786,8 @@ TEST(Any, VoidSelfCopyAssignment) {
     any = *&any;
 
     ASSERT_FALSE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
     ASSERT_EQ(any.type(), entt::type_id<void>());
     ASSERT_EQ(entt::any_cast<int>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
@@ -691,7 +801,8 @@ TEST(Any, VoidMoveConstruction) {
 
     ASSERT_FALSE(any);
     ASSERT_FALSE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::empty);
     ASSERT_EQ(other.type(), entt::type_id<void>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
 }
@@ -705,16 +816,23 @@ TEST(Any, VoidMoveAssignment) {
 
     ASSERT_FALSE(any);
     ASSERT_FALSE(other);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(other.owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::empty);
     ASSERT_EQ(other.type(), entt::type_id<void>());
     ASSERT_EQ(entt::any_cast<double>(&other), nullptr);
 }
 
-ENTT_DEBUG_TEST(AnyDeathTest, VoidSelfMoveAssignment) {
+TEST(AnyDeathTest, VoidSelfMoveAssignment) {
     entt::any any{std::in_place_type<void>};
 
     // avoid warnings due to self-assignment
-    ASSERT_DEATH(any = std::move(*&any), "");
+    any = std::move(*&any);
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
+    ASSERT_EQ(any.type(), entt::type_id<void>());
+    ASSERT_EQ(any.data(), nullptr);
 }
 
 TEST(Any, SBOMoveValidButUnspecifiedState) {
@@ -799,7 +917,8 @@ TEST(Any, Emplace) {
     any.emplace<int>(2);
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
     ASSERT_EQ(any.type(), entt::type_id<int>());
     ASSERT_EQ(entt::any_cast<double>(&any), nullptr);
     ASSERT_EQ(entt::any_cast<int>(any), 2);
@@ -810,7 +929,8 @@ TEST(Any, EmplaceVoid) {
     any.emplace<void>();
 
     ASSERT_FALSE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
     ASSERT_EQ(any.type(), entt::type_id<void>());
 }
 
@@ -818,26 +938,30 @@ TEST(Any, Reset) {
     entt::any any{2};
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
     ASSERT_EQ(any.type(), entt::type_id<int>());
 
     any.reset();
 
     ASSERT_FALSE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
     ASSERT_EQ(any.type(), entt::type_id<void>());
 
     int value = 2;
     any.emplace<int &>(value);
 
     ASSERT_TRUE(any);
+    ASSERT_FALSE(any.owner());
     ASSERT_EQ(any.policy(), entt::any_policy::ref);
     ASSERT_EQ(any.type(), entt::type_id<int>());
 
     any.reset();
 
     ASSERT_FALSE(any);
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(any.owner());
+    ASSERT_EQ(any.policy(), entt::any_policy::empty);
     ASSERT_EQ(any.type(), entt::type_id<void>());
 }
 
@@ -847,8 +971,11 @@ TEST(Any, SBOSwap) {
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
-    ASSERT_EQ(rhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_TRUE(rhs.owner());
+
+    ASSERT_EQ(lhs.policy(), entt::any_policy::embedded);
+    ASSERT_EQ(rhs.policy(), entt::any_policy::embedded);
 
     ASSERT_EQ(lhs.type(), entt::type_id<int>());
     ASSERT_EQ(rhs.type(), entt::type_id<char>());
@@ -864,8 +991,11 @@ TEST(Any, NoSBOSwap) {
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
-    ASSERT_EQ(rhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_TRUE(rhs.owner());
+
+    ASSERT_EQ(lhs.policy(), entt::any_policy::dynamic);
+    ASSERT_EQ(rhs.policy(), entt::any_policy::dynamic);
 
     ASSERT_EQ(entt::any_cast<fat>(lhs), (fat{.4, .3, .2, .1}));
     ASSERT_EQ(entt::any_cast<fat>(rhs), (fat{.1, .2, .3, .4}));
@@ -878,8 +1008,11 @@ TEST(Any, VoidSwap) {
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
-    ASSERT_EQ(rhs.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(lhs.owner());
+    ASSERT_FALSE(rhs.owner());
+
+    ASSERT_EQ(lhs.policy(), entt::any_policy::empty);
+    ASSERT_EQ(rhs.policy(), entt::any_policy::empty);
 
     ASSERT_EQ(pre, lhs.data());
 }
@@ -890,8 +1023,11 @@ TEST(Any, SBOWithNoSBOSwap) {
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
-    ASSERT_EQ(rhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_TRUE(rhs.owner());
+
+    ASSERT_EQ(lhs.policy(), entt::any_policy::embedded);
+    ASSERT_EQ(rhs.policy(), entt::any_policy::dynamic);
 
     ASSERT_EQ(lhs.type(), entt::type_id<char>());
     ASSERT_EQ(rhs.type(), entt::type_id<fat>());
@@ -908,7 +1044,10 @@ TEST(Any, SBOWithRefSwap) {
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_FALSE(rhs.owner());
+
+    ASSERT_EQ(lhs.policy(), entt::any_policy::embedded);
     ASSERT_EQ(rhs.policy(), entt::any_policy::ref);
 
     ASSERT_EQ(lhs.type(), entt::type_id<char>());
@@ -927,7 +1066,10 @@ TEST(Any, SBOWithConstRefSwap) {
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_FALSE(rhs.owner());
+
+    ASSERT_EQ(lhs.policy(), entt::any_policy::embedded);
     ASSERT_EQ(rhs.policy(), entt::any_policy::cref);
 
     ASSERT_EQ(lhs.type(), entt::type_id<char>());
@@ -947,7 +1089,8 @@ TEST(Any, SBOWithEmptySwap) {
     std::swap(lhs, rhs);
 
     ASSERT_FALSE(lhs);
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(lhs.owner());
+    ASSERT_EQ(lhs.policy(), entt::any_policy::empty);
     ASSERT_EQ(rhs.type(), entt::type_id<char>());
     ASSERT_EQ(entt::any_cast<char>(&lhs), nullptr);
     ASSERT_EQ(entt::any_cast<double>(&rhs), nullptr);
@@ -956,7 +1099,8 @@ TEST(Any, SBOWithEmptySwap) {
     std::swap(lhs, rhs);
 
     ASSERT_FALSE(rhs);
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_EQ(lhs.policy(), entt::any_policy::embedded);
     ASSERT_EQ(lhs.type(), entt::type_id<char>());
     ASSERT_EQ(entt::any_cast<double>(&lhs), nullptr);
     ASSERT_EQ(entt::any_cast<char>(&rhs), nullptr);
@@ -970,7 +1114,8 @@ TEST(Any, SBOWithVoidSwap) {
     std::swap(lhs, rhs);
 
     ASSERT_FALSE(lhs);
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(lhs.owner());
+    ASSERT_EQ(lhs.policy(), entt::any_policy::empty);
     ASSERT_EQ(rhs.type(), entt::type_id<char>());
     ASSERT_EQ(entt::any_cast<char>(&lhs), nullptr);
     ASSERT_EQ(entt::any_cast<double>(&rhs), nullptr);
@@ -979,7 +1124,8 @@ TEST(Any, SBOWithVoidSwap) {
     std::swap(lhs, rhs);
 
     ASSERT_FALSE(rhs);
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_EQ(lhs.policy(), entt::any_policy::embedded);
     ASSERT_EQ(lhs.type(), entt::type_id<char>());
     ASSERT_EQ(entt::any_cast<double>(&lhs), nullptr);
     ASSERT_EQ(entt::any_cast<char>(&rhs), nullptr);
@@ -993,7 +1139,10 @@ TEST(Any, NoSBOWithRefSwap) {
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_FALSE(rhs.owner());
+
+    ASSERT_EQ(lhs.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(rhs.policy(), entt::any_policy::ref);
 
     ASSERT_EQ(lhs.type(), entt::type_id<fat>());
@@ -1012,7 +1161,10 @@ TEST(Any, NoSBOWithConstRefSwap) {
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_FALSE(rhs.owner());
+
+    ASSERT_EQ(lhs.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(rhs.policy(), entt::any_policy::cref);
 
     ASSERT_EQ(lhs.type(), entt::type_id<fat>());
@@ -1032,7 +1184,8 @@ TEST(Any, NoSBOWithEmptySwap) {
     std::swap(lhs, rhs);
 
     ASSERT_FALSE(lhs);
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(lhs.owner());
+    ASSERT_EQ(lhs.policy(), entt::any_policy::empty);
     ASSERT_EQ(rhs.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<fat>(&lhs), nullptr);
     ASSERT_EQ(entt::any_cast<double>(&rhs), nullptr);
@@ -1041,7 +1194,8 @@ TEST(Any, NoSBOWithEmptySwap) {
     std::swap(lhs, rhs);
 
     ASSERT_FALSE(rhs);
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_EQ(lhs.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(lhs.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&lhs), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(&rhs), nullptr);
@@ -1055,7 +1209,8 @@ TEST(Any, NoSBOWithVoidSwap) {
     std::swap(lhs, rhs);
 
     ASSERT_FALSE(lhs);
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_FALSE(lhs.owner());
+    ASSERT_EQ(lhs.policy(), entt::any_policy::empty);
     ASSERT_EQ(rhs.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<fat>(&lhs), nullptr);
     ASSERT_EQ(entt::any_cast<double>(&rhs), nullptr);
@@ -1064,7 +1219,8 @@ TEST(Any, NoSBOWithVoidSwap) {
     std::swap(lhs, rhs);
 
     ASSERT_FALSE(rhs);
-    ASSERT_EQ(lhs.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(lhs.owner());
+    ASSERT_EQ(lhs.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(lhs.type(), entt::type_id<fat>());
     ASSERT_EQ(entt::any_cast<double>(&lhs), nullptr);
     ASSERT_EQ(entt::any_cast<fat>(&rhs), nullptr);
@@ -1076,6 +1232,9 @@ TEST(Any, AsRef) {
     auto ref = any.as_ref();
     auto cref = std::as_const(any).as_ref();
 
+    ASSERT_FALSE(ref.owner());
+    ASSERT_FALSE(cref.owner());
+
     ASSERT_EQ(ref.policy(), entt::any_policy::ref);
     ASSERT_EQ(cref.policy(), entt::any_policy::cref);
 
@@ -1110,6 +1269,9 @@ TEST(Any, AsRef) {
 
     std::swap(ref, cref);
 
+    ASSERT_FALSE(ref.owner());
+    ASSERT_FALSE(cref.owner());
+
     ASSERT_EQ(ref.policy(), entt::any_policy::cref);
     ASSERT_EQ(cref.policy(), entt::any_policy::ref);
 
@@ -1119,6 +1281,9 @@ TEST(Any, AsRef) {
     ref = ref.as_ref();
     cref = std::as_const(cref).as_ref();
 
+    ASSERT_FALSE(ref.owner());
+    ASSERT_FALSE(cref.owner());
+
     ASSERT_EQ(ref.policy(), entt::any_policy::cref);
     ASSERT_EQ(cref.policy(), entt::any_policy::cref);
 
@@ -1136,8 +1301,11 @@ TEST(Any, AsRef) {
     ref = 2;
     cref = 2;
 
-    ASSERT_EQ(ref.policy(), entt::any_policy::owner);
-    ASSERT_EQ(cref.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(ref.owner());
+    ASSERT_TRUE(cref.owner());
+
+    ASSERT_EQ(ref.policy(), entt::any_policy::embedded);
+    ASSERT_EQ(cref.policy(), entt::any_policy::embedded);
 
     ASSERT_NE(entt::any_cast<int>(&ref), nullptr);
     ASSERT_NE(entt::any_cast<int>(&cref), nullptr);
@@ -1320,8 +1488,12 @@ TEST(Any, MakeAny) {
     ASSERT_TRUE(ext);
     ASSERT_TRUE(ref);
 
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
-    ASSERT_EQ(ext.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_TRUE(ext.owner());
+    ASSERT_FALSE(ref.owner());
+
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
+    ASSERT_EQ(ext.policy(), entt::any_policy::embedded);
     ASSERT_EQ(ref.policy(), entt::any_policy::ref);
 
     ASSERT_EQ(entt::any_cast<const int &>(any), 2);
@@ -1347,7 +1519,11 @@ TEST(Any, ForwardAsAny) {
     ASSERT_TRUE(ref);
     ASSERT_TRUE(cref);
 
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_FALSE(ref.owner());
+    ASSERT_FALSE(cref.owner());
+
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
     ASSERT_EQ(ref.policy(), entt::any_policy::ref);
     ASSERT_EQ(cref.policy(), entt::any_policy::cref);
 
@@ -1371,7 +1547,10 @@ TEST(Any, NonCopyableType) {
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
 
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_FALSE(other.owner());
+
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
     ASSERT_EQ(other.policy(), entt::any_policy::cref);
     ASSERT_EQ(any.type(), other.type());
 
@@ -1383,16 +1562,22 @@ TEST(Any, NonCopyableType) {
     ASSERT_TRUE(any);
     ASSERT_FALSE(copy);
 
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
-    ASSERT_EQ(copy.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_FALSE(copy.owner());
+
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
+    ASSERT_EQ(copy.policy(), entt::any_policy::empty);
 
     copy = any;
 
     ASSERT_TRUE(any);
     ASSERT_FALSE(copy);
 
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
-    ASSERT_EQ(copy.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_FALSE(copy.owner());
+
+    ASSERT_EQ(any.policy(), entt::any_policy::embedded);
+    ASSERT_EQ(copy.policy(), entt::any_policy::empty);
 }
 
 TEST(Any, NonCopyableValueType) {
@@ -1419,8 +1604,11 @@ TEST(Any, NonMovableType) {
     ASSERT_TRUE(any);
     ASSERT_TRUE(other);
 
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
-    ASSERT_EQ(other.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_TRUE(other.owner());
+
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
+    ASSERT_EQ(other.policy(), entt::any_policy::dynamic);
     ASSERT_EQ(any.type(), other.type());
 
     ASSERT_TRUE(any.assign(other));
@@ -1431,16 +1619,22 @@ TEST(Any, NonMovableType) {
     ASSERT_TRUE(any);
     ASSERT_TRUE(copy);
 
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
-    ASSERT_EQ(copy.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_TRUE(copy.owner());
+
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
+    ASSERT_EQ(copy.policy(), entt::any_policy::dynamic);
 
     copy = any;
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(copy);
 
-    ASSERT_EQ(any.policy(), entt::any_policy::owner);
-    ASSERT_EQ(copy.policy(), entt::any_policy::owner);
+    ASSERT_TRUE(any.owner());
+    ASSERT_TRUE(copy.owner());
+
+    ASSERT_EQ(any.policy(), entt::any_policy::dynamic);
+    ASSERT_EQ(copy.policy(), entt::any_policy::dynamic);
 }
 
 TEST(Any, Array) {
@@ -1477,8 +1671,11 @@ TEST(Any, CopyMoveReference) {
     ASSERT_TRUE(move);
     ASSERT_TRUE(copy);
 
+    ASSERT_FALSE(move.owner());
+    ASSERT_TRUE(copy.owner());
+
     ASSERT_EQ(move.policy(), entt::any_policy::ref);
-    ASSERT_EQ(copy.policy(), entt::any_policy::owner);
+    ASSERT_EQ(copy.policy(), entt::any_policy::embedded);
 
     ASSERT_EQ(move.type(), entt::type_id<int>());
     ASSERT_EQ(copy.type(), entt::type_id<int>());
@@ -1507,8 +1704,11 @@ TEST(Any, CopyMoveConstReference) {
     ASSERT_TRUE(move);
     ASSERT_TRUE(copy);
 
+    ASSERT_FALSE(move.owner());
+    ASSERT_TRUE(copy.owner());
+
     ASSERT_EQ(move.policy(), entt::any_policy::cref);
-    ASSERT_EQ(copy.policy(), entt::any_policy::owner);
+    ASSERT_EQ(copy.policy(), entt::any_policy::embedded);
 
     ASSERT_EQ(move.type(), entt::type_id<int>());
     ASSERT_EQ(copy.type(), entt::type_id<int>());
