@@ -1,16 +1,19 @@
 #include <algorithm>
+#include <array>
 #include <cstddef>
 #include <functional>
-#include <iterator>
 #include <tuple>
-#include <type_traits>
 #include <utility>
 #include <gtest/gtest.h>
+#include <entt/entity/entity.hpp>
 #include <entt/entity/group.hpp>
+#include <entt/entity/mixin.hpp>
 #include <entt/entity/registry.hpp>
-#include "../common/boxed_int.h"
-#include "../common/config.h"
-#include "../common/empty.h"
+#include <entt/entity/view.hpp>
+#include <entt/signal/sigh.hpp>
+#include "../../common/boxed_type.h"
+#include "../../common/config.h"
+#include "../../common/empty.h"
 
 TEST(NonOwningGroup, Functionalities) {
     entt::registry registry;
@@ -23,7 +26,7 @@ TEST(NonOwningGroup, Functionalities) {
     registry.emplace<char>(e0, '1');
 
     const auto e1 = registry.create();
-    registry.emplace<int>(e1, 42); // NOLINT
+    registry.emplace<int>(e1, 4);
     registry.emplace<char>(e1, '2');
 
     ASSERT_FALSE(group.empty());
@@ -47,7 +50,7 @@ TEST(NonOwningGroup, Functionalities) {
     ASSERT_EQ(group.size(), 1u);
 
     for(auto entity: group) {
-        ASSERT_EQ(std::get<0>(cgroup.get<const int, const char>(entity)), 42);
+        ASSERT_EQ(std::get<0>(cgroup.get<const int, const char>(entity)), 4);
         ASSERT_EQ(std::get<1>(group.get<int, char>(entity)), '2');
         ASSERT_EQ(cgroup.get<1>(entity), '2');
     }
@@ -175,7 +178,7 @@ TEST(NonOwningGroup, Empty) {
 
 TEST(NonOwningGroup, Each) {
     entt::registry registry;
-    const entt::entity entity[2]{registry.create(), registry.create()}; // NOLINT
+    std::array entity{registry.create(), registry.create()};
 
     auto group = registry.group(entt::get<int, char>);
     auto cgroup = std::as_const(registry).group_if_exists(entt::get<const int, const char>);
@@ -318,8 +321,9 @@ TEST(NonOwningGroup, SortAsAPool) {
         ASSERT_EQ(group.get<const int>(entity), --ival);
     }
 
-    registry.sort<unsigned int>(std::less<unsigned int>{});
-    group.sort_as(*group.storage<unsigned int>()); // NOLINT
+    registry.sort<unsigned int>(std::less{});
+    const entt::sparse_set &other = *group.storage<unsigned int>();
+    group.sort_as(other.begin(), other.end());
 
     ASSERT_EQ((group.get<const int, unsigned int>(e0)), (std::make_tuple(0, 0u)));
     ASSERT_EQ((group.get<0, 1>(e1)), (std::make_tuple(1, 1u)));
@@ -347,7 +351,7 @@ TEST(NonOwningGroup, IndexRebuiltOnDestroy) {
     registry.emplace<int>(e1, 1);
 
     registry.destroy(e0);
-    registry.emplace<int>(registry.create(), 42); // NOLINT
+    registry.emplace<int>(registry.create(), 4);
 
     ASSERT_EQ(group.size(), 1u);
     ASSERT_EQ(group[{}], e1);
@@ -398,12 +402,12 @@ TEST(NonOwningGroup, ConstNonConstAndAllInBetween) {
     testing::StaticAssertTypeEq<decltype(std::as_const(registry).group_if_exists(entt::get<const int, char>)), decltype(std::as_const(registry).group_if_exists(entt::get<const int, const char>))>();
     testing::StaticAssertTypeEq<decltype(std::as_const(registry).group_if_exists(entt::get<int, const char>)), decltype(std::as_const(registry).group_if_exists(entt::get<const int, const char>))>();
 
-    group.each([](auto &&i, auto &&c) {
-        testing::StaticAssertTypeEq<decltype(i), int &>();
-        testing::StaticAssertTypeEq<decltype(c), const char &>();
+    group.each([](auto &&iv, auto &&cv) {
+        testing::StaticAssertTypeEq<decltype(iv), int &>();
+        testing::StaticAssertTypeEq<decltype(cv), const char &>();
     });
 
-    for(auto [entt, iv, cv]: group.each()) {
+    for([[maybe_unused]] auto [entt, iv, cv]: group.each()) {
         testing::StaticAssertTypeEq<decltype(entt), entt::entity>();
         testing::StaticAssertTypeEq<decltype(iv), int &>();
         testing::StaticAssertTypeEq<decltype(cv), const char &>();
@@ -643,12 +647,12 @@ TEST(NonOwningGroup, ExtendedGet) {
     entt::registry registry;
     const auto entity = registry.create();
 
-    registry.emplace<int>(entity, 42); // NOLINT
+    registry.emplace<int>(entity, 3);
     registry.emplace<char>(entity, 'c');
 
     const auto tup = registry.group(entt::get<int, char>).get(entity);
 
-    ASSERT_EQ(std::get<0>(tup), 42);
+    ASSERT_EQ(std::get<0>(tup), 3);
     ASSERT_EQ(std::get<1>(tup), 'c');
 }
 
@@ -754,7 +758,7 @@ TEST(NonOwningGroup, Overlapping) {
     ASSERT_FALSE(group.empty());
     ASSERT_TRUE(other.empty());
 
-    registry.emplace<int>(entity, 42); // NOLINT
+    registry.emplace<int>(entity, 2);
 
     ASSERT_FALSE(group.empty());
     ASSERT_FALSE(other.empty());
@@ -776,7 +780,7 @@ TEST(OwningGroup, Functionalities) {
     registry.emplace<char>(e0, '1');
 
     const auto e1 = registry.create();
-    registry.emplace<int>(e1, 42); // NOLINT
+    registry.emplace<int>(e1, 4);
     registry.emplace<char>(e1, '2');
 
     ASSERT_FALSE(group.empty());
@@ -799,17 +803,17 @@ TEST(OwningGroup, Functionalities) {
 
     ASSERT_EQ(group.size(), 1u);
 
-    ASSERT_EQ(cgroup.storage<const int>()->raw()[0u][0u], 42);
-    ASSERT_EQ(group.storage<int>()->raw()[0u][0u], 42);
+    ASSERT_EQ(cgroup.storage<const int>()->raw()[0u][0u], 4);
+    ASSERT_EQ(group.storage<int>()->raw()[0u][0u], 4);
 
     for(auto entity: group) {
-        ASSERT_EQ(std::get<0>(cgroup.get<const int, const char>(entity)), 42);
+        ASSERT_EQ(std::get<0>(cgroup.get<const int, const char>(entity)), 4);
         ASSERT_EQ(std::get<1>(group.get<int, char>(entity)), '2');
         ASSERT_EQ(cgroup.get<1>(entity), '2');
     }
 
     ASSERT_EQ(group.handle().data()[0u], e1);
-    ASSERT_EQ(group.storage<int>()->raw()[0u][0u], 42);
+    ASSERT_EQ(group.storage<int>()->raw()[0u][0u], 4);
 
     registry.erase<char>(e0);
     registry.erase<char>(e1);
@@ -924,7 +928,7 @@ TEST(OwningGroup, Empty) {
 
 TEST(OwningGroup, Each) {
     entt::registry registry;
-    const entt::entity entity[2]{registry.create(), registry.create()}; // NOLINT
+    std::array entity{registry.create(), registry.create()};
 
     auto group = registry.group<int>(entt::get<char>);
     auto cgroup = std::as_const(registry).group_if_exists<const int>(entt::get<const char>);
@@ -978,20 +982,13 @@ TEST(OwningGroup, SortOrdered) {
     entt::registry registry;
     auto group = registry.group<test::boxed_int, char>();
 
-    entt::entity entity[5]{}; // NOLINT
-    registry.create(std::begin(entity), std::end(entity));
+    const std::array value{test::boxed_int{16}, test::boxed_int{8}, test::boxed_int{4}, test::boxed_int{1}, test::boxed_int{2}};
+    std::array<entt::entity, value.size()> entity{};
+    const std::array other{'a', 'b', 'c'};
 
-    registry.emplace<test::boxed_int>(entity[0], 12); // NOLINT
-    registry.emplace<char>(entity[0], 'a');
-
-    registry.emplace<test::boxed_int>(entity[1], 9); // NOLINT
-    registry.emplace<char>(entity[1], 'b');
-
-    registry.emplace<test::boxed_int>(entity[2], 6); // NOLINT
-    registry.emplace<char>(entity[2], 'c');
-
-    registry.emplace<test::boxed_int>(entity[3], 1);
-    registry.emplace<test::boxed_int>(entity[4], 2);
+    registry.create(entity.begin(), entity.end());
+    registry.insert<test::boxed_int>(entity.begin(), entity.end(), value.begin());
+    registry.insert<char>(entity.begin(), entity.begin() + other.size(), other.begin());
 
     group.sort([&group](const entt::entity lhs, const entt::entity rhs) {
         return group.get<test::boxed_int>(lhs).value < group.get<0>(rhs).value;
@@ -1003,19 +1000,19 @@ TEST(OwningGroup, SortOrdered) {
     ASSERT_EQ(group.handle().data()[3u], entity[3]);
     ASSERT_EQ(group.handle().data()[4u], entity[4]);
 
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][0u].value, 12);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][1u].value, 9);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][2u].value, 6);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][3u].value, 1);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][4u].value, 2);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][0u], value[0]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][1u], value[1]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][2u], value[2]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][3u], value[3]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][4u], value[4]);
 
-    ASSERT_EQ(group.storage<char>()->raw()[0u][0u], 'a');
-    ASSERT_EQ(group.storage<char>()->raw()[0u][1u], 'b');
-    ASSERT_EQ(group.storage<char>()->raw()[0u][2u], 'c');
+    ASSERT_EQ(group.storage<char>()->raw()[0u][0u], other[0]);
+    ASSERT_EQ(group.storage<char>()->raw()[0u][1u], other[1]);
+    ASSERT_EQ(group.storage<char>()->raw()[0u][2u], other[2]);
 
-    ASSERT_EQ((group.get<test::boxed_int, char>(entity[0])), (std::make_tuple(test::boxed_int{12}, 'a')));
-    ASSERT_EQ((group.get<0, 1>(entity[1])), (std::make_tuple(test::boxed_int{9}, 'b')));
-    ASSERT_EQ((group.get<test::boxed_int, char>(entity[2])), (std::make_tuple(test::boxed_int{6}, 'c')));
+    ASSERT_EQ((group.get<test::boxed_int, char>(entity[0])), (std::make_tuple(value[0], other[0])));
+    ASSERT_EQ((group.get<0, 1>(entity[1])), (std::make_tuple(value[1], other[1])));
+    ASSERT_EQ((group.get<test::boxed_int, char>(entity[2])), (std::make_tuple(value[2], other[2])));
 
     ASSERT_FALSE(group.contains(entity[3]));
     ASSERT_FALSE(group.contains(entity[4]));
@@ -1025,20 +1022,13 @@ TEST(OwningGroup, SortReverse) {
     entt::registry registry;
     auto group = registry.group<test::boxed_int, char>();
 
-    entt::entity entity[5]{}; // NOLINT
-    registry.create(std::begin(entity), std::end(entity));
+    const std::array value{test::boxed_int{4}, test::boxed_int{8}, test::boxed_int{16}, test::boxed_int{1}, test::boxed_int{2}};
+    std::array<entt::entity, value.size()> entity{};
+    const std::array other{'a', 'b', 'c'};
 
-    registry.emplace<test::boxed_int>(entity[0], 6); // NOLINT
-    registry.emplace<char>(entity[0], 'a');
-
-    registry.emplace<test::boxed_int>(entity[1], 9); // NOLINT
-    registry.emplace<char>(entity[1], 'b');
-
-    registry.emplace<test::boxed_int>(entity[2], 12); // NOLINT
-    registry.emplace<char>(entity[2], 'c');
-
-    registry.emplace<test::boxed_int>(entity[3], 1);
-    registry.emplace<test::boxed_int>(entity[4], 2);
+    registry.create(entity.begin(), entity.end());
+    registry.insert<test::boxed_int>(entity.begin(), entity.end(), value.begin());
+    registry.insert<char>(entity.begin(), entity.begin() + other.size(), other.begin());
 
     group.sort<test::boxed_int>([](const auto &lhs, const auto &rhs) {
         return lhs.value < rhs.value;
@@ -1050,19 +1040,19 @@ TEST(OwningGroup, SortReverse) {
     ASSERT_EQ(group.handle().data()[3u], entity[3]);
     ASSERT_EQ(group.handle().data()[4u], entity[4]);
 
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][0u].value, 12);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][1u].value, 9);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][2u].value, 6);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][3u].value, 1);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][4u].value, 2);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][0u], value[2]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][1u], value[1]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][2u], value[0]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][3u], value[3]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][4u], value[4]);
 
-    ASSERT_EQ(group.storage<char>()->raw()[0u][0u], 'c');
-    ASSERT_EQ(group.storage<char>()->raw()[0u][1u], 'b');
-    ASSERT_EQ(group.storage<char>()->raw()[0u][2u], 'a');
+    ASSERT_EQ(group.storage<char>()->raw()[0u][0u], other[2]);
+    ASSERT_EQ(group.storage<char>()->raw()[0u][1u], other[1]);
+    ASSERT_EQ(group.storage<char>()->raw()[0u][2u], other[0]);
 
-    ASSERT_EQ((group.get<test::boxed_int, char>(entity[0])), (std::make_tuple(test::boxed_int{6}, 'a')));
-    ASSERT_EQ((group.get<0, 1>(entity[1])), (std::make_tuple(test::boxed_int{9}, 'b')));
-    ASSERT_EQ((group.get<test::boxed_int, char>(entity[2])), (std::make_tuple(test::boxed_int{12}, 'c')));
+    ASSERT_EQ((group.get<test::boxed_int, char>(entity[0])), (std::make_tuple(value[0], other[0])));
+    ASSERT_EQ((group.get<0, 1>(entity[1])), (std::make_tuple(value[1], other[1])));
+    ASSERT_EQ((group.get<test::boxed_int, char>(entity[2])), (std::make_tuple(value[2], other[2])));
 
     ASSERT_FALSE(group.contains(entity[3]));
     ASSERT_FALSE(group.contains(entity[4]));
@@ -1072,26 +1062,13 @@ TEST(OwningGroup, SortUnordered) {
     entt::registry registry;
     auto group = registry.group<test::boxed_int>(entt::get<char>);
 
-    entt::entity entity[7]{}; // NOLINT
-    registry.create(std::begin(entity), std::end(entity));
+    const std::array value{test::boxed_int{16}, test::boxed_int{2}, test::boxed_int{1}, test::boxed_int{32}, test::boxed_int{64}, test::boxed_int{4}, test::boxed_int{8}};
+    std::array<entt::entity, value.size()> entity{};
+    const std::array other{'c', 'b', 'a', 'd', 'e'};
 
-    registry.emplace<test::boxed_int>(entity[0], 6); // NOLINT
-    registry.emplace<char>(entity[0], 'c');
-
-    registry.emplace<test::boxed_int>(entity[1], 3);
-    registry.emplace<char>(entity[1], 'b');
-
-    registry.emplace<test::boxed_int>(entity[2], 1);
-    registry.emplace<char>(entity[2], 'a');
-
-    registry.emplace<test::boxed_int>(entity[3], 9); // NOLINT
-    registry.emplace<char>(entity[3], 'd');
-
-    registry.emplace<test::boxed_int>(entity[4], 12); // NOLINT
-    registry.emplace<char>(entity[4], 'e');
-
-    registry.emplace<test::boxed_int>(entity[5], 4); // NOLINT
-    registry.emplace<test::boxed_int>(entity[6], 5); // NOLINT
+    registry.create(entity.begin(), entity.end());
+    registry.insert<test::boxed_int>(entity.begin(), entity.end(), value.begin());
+    registry.insert<char>(entity.begin(), entity.begin() + other.size(), other.begin());
 
     group.sort<test::boxed_int, char>([](const auto lhs, const auto rhs) {
         testing::StaticAssertTypeEq<decltype(std::get<0>(lhs)), test::boxed_int &>();
@@ -1107,19 +1084,19 @@ TEST(OwningGroup, SortUnordered) {
     ASSERT_EQ(group.handle().data()[5u], entity[5]);
     ASSERT_EQ(group.handle().data()[6u], entity[6]);
 
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][0u].value, 12);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][1u].value, 9);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][2u].value, 6);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][3u].value, 3);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][4u].value, 1);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][5u].value, 4);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][6u].value, 5);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][0u], value[4]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][1u], value[3]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][2u], value[0]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][3u], value[1]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][4u], value[2]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][5u], value[5]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][6u], value[6]);
 
-    ASSERT_EQ(group.get<char>(group.handle().data()[0u]), 'e');
-    ASSERT_EQ(group.get<1>(group.handle().data()[1u]), 'd');
-    ASSERT_EQ(group.get<char>(group.handle().data()[2u]), 'c');
-    ASSERT_EQ(group.get<1>(group.handle().data()[3u]), 'b');
-    ASSERT_EQ(group.get<char>(group.handle().data()[4u]), 'a');
+    ASSERT_EQ(group.get<char>(group.handle().data()[0u]), other[4]);
+    ASSERT_EQ(group.get<1>(group.handle().data()[1u]), other[3]);
+    ASSERT_EQ(group.get<char>(group.handle().data()[2u]), other[0]);
+    ASSERT_EQ(group.get<1>(group.handle().data()[3u]), other[1]);
+    ASSERT_EQ(group.get<char>(group.handle().data()[4u]), other[2]);
 
     ASSERT_FALSE(group.contains(entity[5]));
     ASSERT_FALSE(group.contains(entity[6]));
@@ -1129,15 +1106,11 @@ TEST(OwningGroup, SortWithExclusionList) {
     entt::registry registry;
     auto group = registry.group<test::boxed_int>(entt::get<>, entt::exclude<char>);
 
-    entt::entity entity[5]{}; // NOLINT
-    registry.create(std::begin(entity), std::end(entity));
-
-    registry.emplace<test::boxed_int>(entity[0], 0);
-    registry.emplace<test::boxed_int>(entity[1], 1);
-    registry.emplace<test::boxed_int>(entity[2], 2);
-    registry.emplace<test::boxed_int>(entity[3], 3);
-    registry.emplace<test::boxed_int>(entity[4], 4);
+    const std::array value{test::boxed_int{1}, test::boxed_int{2}, test::boxed_int{4}, test::boxed_int{8}, test::boxed_int{16}};
+    std::array<entt::entity, value.size()> entity{};
 
+    registry.create(entity.begin(), entity.end());
+    registry.insert<test::boxed_int>(entity.begin(), entity.end(), value.begin());
     registry.emplace<char>(entity[2]);
 
     group.sort([](const entt::entity lhs, const entt::entity rhs) {
@@ -1149,15 +1122,15 @@ TEST(OwningGroup, SortWithExclusionList) {
     ASSERT_EQ(group.handle().data()[2u], entity[1]);
     ASSERT_EQ(group.handle().data()[3u], entity[0]);
 
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][0u].value, 4);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][1u].value, 3);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][2u].value, 1);
-    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][3u].value, 0);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][0u], value[4]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][1u], value[3]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][2u], value[1]);
+    ASSERT_EQ(group.storage<test::boxed_int>()->raw()[0u][3u], value[0]);
 
-    ASSERT_EQ(group.get<test::boxed_int>(entity[0]).value, 0);
-    ASSERT_EQ(group.get<0>(entity[1]).value, 1);
-    ASSERT_EQ(group.get<test::boxed_int>(entity[3]).value, 3);
-    ASSERT_EQ(group.get<0>(entity[4]).value, 4);
+    ASSERT_EQ(group.get<test::boxed_int>(entity[0]), value[0]);
+    ASSERT_EQ(group.get<0>(entity[1]), value[1]);
+    ASSERT_EQ(group.get<test::boxed_int>(entity[3]), value[3]);
+    ASSERT_EQ(group.get<0>(entity[4]), value[4]);
 
     ASSERT_FALSE(group.contains(entity[2]));
 }
@@ -1176,7 +1149,7 @@ TEST(OwningGroup, IndexRebuiltOnDestroy) {
     registry.emplace<int>(e1, 1);
 
     registry.destroy(e0);
-    registry.emplace<int>(registry.create(), 42); // NOLINT
+    registry.emplace<int>(registry.create(), 4);
 
     ASSERT_EQ(group.size(), 1u);
     ASSERT_EQ(group[{}], e1);
@@ -1235,14 +1208,14 @@ TEST(OwningGroup, ConstNonConstAndAllInBetween) {
     testing::StaticAssertTypeEq<decltype(std::as_const(registry).group_if_exists<const int>(entt::get<char>)), decltype(std::as_const(registry).group_if_exists<const int>(entt::get<const char>))>();
     testing::StaticAssertTypeEq<decltype(std::as_const(registry).group_if_exists<int>(entt::get<const char>)), decltype(std::as_const(registry).group_if_exists<const int>(entt::get<const char>))>();
 
-    group.each([](auto &&i, auto &&c, auto &&d, auto &&f) {
-        testing::StaticAssertTypeEq<decltype(i), int &>();
-        testing::StaticAssertTypeEq<decltype(c), const char &>();
-        testing::StaticAssertTypeEq<decltype(d), double &>();
-        testing::StaticAssertTypeEq<decltype(f), const float &>();
+    group.each([](auto &&iv, auto &&cv, auto &&dv, auto &&fv) {
+        testing::StaticAssertTypeEq<decltype(iv), int &>();
+        testing::StaticAssertTypeEq<decltype(cv), const char &>();
+        testing::StaticAssertTypeEq<decltype(dv), double &>();
+        testing::StaticAssertTypeEq<decltype(fv), const float &>();
     });
 
-    for(auto [entt, iv, cv, dv, fv]: group.each()) {
+    for([[maybe_unused]] auto [entt, iv, cv, dv, fv]: group.each()) {
         testing::StaticAssertTypeEq<decltype(entt), entt::entity>();
         testing::StaticAssertTypeEq<decltype(iv), int &>();
         testing::StaticAssertTypeEq<decltype(cv), const char &>();
@@ -1475,11 +1448,12 @@ TEST(OwningGroup, SignalRace) {
 
 TEST(OwningGroup, StableLateInitialization) {
     entt::registry registry;
+    constexpr auto number_of_entities = 30u;
 
-    for(std::size_t i{}; i < 30u; ++i) { // NOLINT
+    for(std::size_t i{}; i < number_of_entities; ++i) {
         auto entity = registry.create();
-        if(!(i % 2u)) registry.emplace<int>(entity);
-        if(!(i % 3u)) registry.emplace<char>(entity);
+        if((i % 2u) == 0u) { registry.emplace<int>(entity); }
+        if((i % 3u) == 0u) { registry.emplace<char>(entity); }
     }
 
     // thanks to @pgruenbacher for pointing out this corner case
@@ -1496,10 +1470,10 @@ TEST(OwningGroup, PreventEarlyOptOut) {
     registry.emplace<int>(entity, 2);
 
     // thanks to @pgruenbacher for pointing out this corner case
-    registry.group<char, int>().each([entity](const auto entt, const auto &c, const auto &i) {
+    registry.group<char, int>().each([entity](const auto entt, const auto &cv, const auto &iv) {
         ASSERT_EQ(entity, entt);
-        ASSERT_EQ(c, 'c');
-        ASSERT_EQ(i, 2);
+        ASSERT_EQ(cv, 'c');
+        ASSERT_EQ(iv, 2);
     });
 }
 
@@ -1551,12 +1525,12 @@ TEST(OwningGroup, ExtendedGet) {
     entt::registry registry;
     const auto entity = registry.create();
 
-    registry.emplace<int>(entity, 42); // NOLINT
+    registry.emplace<int>(entity, 3);
     registry.emplace<char>(entity, 'c');
 
     const auto tup = registry.group<int>(entt::get<char>).get(entity);
 
-    ASSERT_EQ(std::get<0>(tup), 42);
+    ASSERT_EQ(std::get<0>(tup), 3);
     ASSERT_EQ(std::get<1>(tup), 'c');
 }
 
