@@ -10,6 +10,7 @@
 #include <entt/entity/mixin.hpp>
 #include <entt/entity/registry.hpp>
 #include <entt/entity/storage.hpp>
+#include <entt/signal/sigh.hpp>
 #include "../../common/config.h"
 #include "../../common/empty.h"
 #include "../../common/entity.h"
@@ -31,7 +32,7 @@ void remove(Type &storage, const typename Type::registry_type &, const typename
 
 template<typename Type>
 struct entt::storage_type<Type, test::entity, std::allocator<Type>, std::enable_if_t<!std::is_same_v<Type, test::entity>>> {
-    using type = entt::basic_sigh_mixin<entt::basic_storage<Type, test::entity>, test::basic_custom_registry<test::entity>>;
+    using type = entt::basic_sigh_mixin<entt::basic_storage<Type, test::entity>, test::custom_registry<test::entity>>;
 };
 
 template<typename Type>
@@ -454,7 +455,7 @@ ENTT_DEBUG_TYPED_TEST(ReactiveMixinDeathTest, Registry) {
 
 TYPED_TEST(ReactiveMixin, CustomRegistry) {
     using value_type = typename TestFixture::type;
-    using registry_type = test::basic_custom_registry<test::entity>;
+    using registry_type = test::custom_registry<test::entity>;
 
     registry_type registry;
     entt::basic_reactive_mixin<entt::basic_storage<value_type, test::entity>, registry_type> pool;
@@ -476,7 +477,7 @@ TYPED_TEST(ReactiveMixin, CustomRegistry) {
 
 ENTT_DEBUG_TYPED_TEST(ReactiveMixinDeathTest, CustomRegistry) {
     using value_type = typename TestFixture::type;
-    using registry_type = test::basic_custom_registry<test::entity>;
+    using registry_type = test::custom_registry<test::entity>;
     entt::basic_reactive_mixin<entt::basic_storage<value_type, test::entity>, registry_type> pool;
     ASSERT_DEATH([[maybe_unused]] auto &registry = pool.registry(), "");
     ASSERT_DEATH([[maybe_unused]] const auto &registry = std::as_const(pool).registry(), "");
@@ -534,6 +535,40 @@ ENTT_DEBUG_TYPED_TEST(ReactiveMixinDeathTest, View) {
     ASSERT_DEATH([[maybe_unused]] const auto cview = std::as_const(pool).view(), "");
 }
 
+TYPED_TEST(ReactiveMixin, AutoDisconnection) {
+    using value_type = typename TestFixture::type;
+
+    entt::registry registry;
+    entt::reactive_mixin<entt::storage<value_type>> pool;
+    const std::array entity{registry.create(), registry.create(), registry.create()};
+
+    ASSERT_TRUE(pool.empty());
+
+    ASSERT_TRUE(registry.on_construct<test::empty>().empty());
+    ASSERT_TRUE(registry.on_update<test::empty>().empty());
+    ASSERT_TRUE(registry.on_destroy<test::empty>().empty());
+
+    pool.bind(registry);
+    pool.template on_construct<test::empty>();
+    pool.template on_update<test::empty>();
+    pool.template on_destroy<test::empty>();
+    registry.emplace<test::empty>(entity[0u]);
+
+    ASSERT_FALSE(pool.empty());
+
+    ASSERT_FALSE(registry.on_construct<test::empty>().empty());
+    ASSERT_FALSE(registry.on_update<test::empty>().empty());
+    ASSERT_FALSE(registry.on_destroy<test::empty>().empty());
+
+    pool.reset();
+
+    ASSERT_FALSE(pool.empty());
+
+    ASSERT_TRUE(registry.on_construct<test::empty>().empty());
+    ASSERT_TRUE(registry.on_update<test::empty>().empty());
+    ASSERT_TRUE(registry.on_destroy<test::empty>().empty());
+}
+
 TYPED_TEST(ReactiveMixin, CustomAllocator) {
     using value_type = typename TestFixture::type;
     using storage_type = entt::reactive_mixin<entt::basic_storage<value_type, entt::entity, test::throwing_allocator<value_type>>>;
@@ -571,6 +606,7 @@ TYPED_TEST(ReactiveMixin, CustomAllocator) {
     ASSERT_NE(pool.capacity(), 0u);
     ASSERT_EQ(pool.size(), 2u);
 
+    other = {};
     pool.swap(other);
     pool = std::move(other);
     test::is_initialized(other);
