@@ -1,65 +1,91 @@
+#include <array>
 #include <cstddef>
-#include <iterator>
 #include <memory>
 #include <type_traits>
 #include <utility>
 #include <gtest/gtest.h>
-#include <entt/core/any.hpp>
 #include <entt/core/type_info.hpp>
 #include <entt/entity/component.hpp>
 #include <entt/entity/entity.hpp>
 #include <entt/entity/mixin.hpp>
 #include <entt/entity/registry.hpp>
 #include <entt/entity/storage.hpp>
-#include "../common/custom_entity.h"
-#include "../common/non_default_constructible.h"
-#include "../common/pointer_stable.h"
-#include "../common/throwing_allocator.hpp"
-#include "../common/throwing_type.hpp"
+#include <entt/signal/sigh.hpp>
+#include "../../common/config.h"
+#include "../../common/entity.h"
+#include "../../common/linter.hpp"
+#include "../../common/non_default_constructible.h"
+#include "../../common/pointer_stable.h"
+#include "../../common/registry.h"
+#include "../../common/throwing_allocator.hpp"
+#include "../../common/throwing_type.hpp"
+
+struct auto_signal final {
+    auto_signal(bool &cflag, bool &uflag, bool &dflag)
+        : created{&cflag},
+          updated{&uflag},
+          destroyed{&dflag} {}
+
+    static void on_construct(entt::registry &registry, const entt::entity entt) {
+        *registry.get<auto_signal>(entt).created = true;
+    }
+
+    static void on_update(entt::registry &registry, const entt::entity entt) {
+        *registry.get<auto_signal>(entt).updated = true;
+    }
+
+    static void on_destroy(entt::registry &registry, const entt::entity entt) {
+        *registry.get<auto_signal>(entt).destroyed = true;
+    }
+
+private:
+    bool *created{};
+    bool *updated{};
+    bool *destroyed{};
+};
 
 template<typename Registry>
 void listener(std::size_t &counter, Registry &, typename Registry::entity_type) {
     ++counter;
 }
 
-struct custom_registry: entt::basic_registry<test::custom_entity> {};
-
-template<typename Type>
-struct entt::storage_type<Type, test::custom_entity, std::allocator<Type>, std::enable_if_t<!std::is_same_v<Type, test::custom_entity>>> {
-    using type = entt::basic_sigh_mixin<entt::basic_storage<Type, test::custom_entity>, custom_registry>;
-};
-
 template<typename Type>
 struct SighMixin: testing::Test {
     using type = Type;
 };
 
+template<typename Type>
+using SighMixinDeathTest = SighMixin<Type>;
+
 using SighMixinTypes = ::testing::Types<int, test::pointer_stable>;
 
 TYPED_TEST_SUITE(SighMixin, SighMixinTypes, );
+TYPED_TEST_SUITE(SighMixinDeathTest, SighMixinTypes, );
+
+TYPED_TEST(SighMixin, Functionalities) {
+    using value_type = typename TestFixture::type;
+    using traits_type = entt::component_traits<value_type>;
 
-TEST(SighMixin, GenericType) {
-    entt::entity entity[2u]{entt::entity{3}, entt::entity{42}}; // NOLINT
-    entt::sigh_mixin<entt::storage<int>> pool;
     entt::registry registry;
+    auto &pool = registry.storage<value_type>();
+    const std::array entity{registry.create(), registry.create()};
+
+    testing::StaticAssertTypeEq<decltype(pool), entt::sigh_mixin<entt::storage<value_type>> &>();
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
-
     ASSERT_EQ(pool.size(), 0u);
 
-    pool.insert(entity, entity + 1u); // NOLINT
+    pool.insert(entity.begin(), entity.begin() + 1u);
     pool.erase(entity[0u]);
 
-    ASSERT_EQ(pool.size(), 0u);
-
+    ASSERT_EQ(pool.size(), traits_type::in_place_delete);
     ASSERT_EQ(on_construct, 0u);
     ASSERT_EQ(on_destroy, 0u);
 
-    pool.on_construct().connect<&listener<entt::registry>>(on_construct);
-    pool.on_destroy().connect<&listener<entt::registry>>(on_destroy);
+    pool.on_construct().template connect<&listener<entt::registry>>(on_construct);
+    pool.on_destroy().template connect<&listener<entt::registry>>(on_destroy);
 
     ASSERT_NE(pool.push(entity[0u]), pool.entt::sparse_set::end());
 
@@ -69,104 +95,41 @@ TEST(SighMixin, GenericType) {
     ASSERT_EQ(on_destroy, 0u);
     ASSERT_EQ(pool.size(), 2u);
 
-    ASSERT_EQ(pool.get(entity[0u]), 0);
-    ASSERT_EQ(pool.get(entity[1u]), 0);
+    ASSERT_EQ(pool.get(entity[0u]), value_type{0});
+    ASSERT_EQ(pool.get(entity[1u]), value_type{0});
 
-    pool.erase(std::begin(entity), std::end(entity));
+    pool.erase(entity.begin(), entity.end());
 
     ASSERT_EQ(on_construct, 2u);
     ASSERT_EQ(on_destroy, 2u);
-    ASSERT_EQ(pool.size(), 0u);
+    ASSERT_EQ(pool.size(), 2u * traits_type::in_place_delete);
 
-    ASSERT_NE(pool.push(std::begin(entity), std::end(entity)), pool.entt::sparse_set::end());
+    ASSERT_NE(pool.push(entity.begin(), entity.end()), pool.entt::sparse_set::end());
 
-    ASSERT_EQ(pool.get(entity[0u]), 0);
-    ASSERT_EQ(pool.get(entity[1u]), 0);
-    ASSERT_EQ(pool.size(), 2u);
+    ASSERT_EQ(pool.get(entity[0u]), value_type{0});
+    ASSERT_EQ(pool.get(entity[1u]), value_type{0});
+    ASSERT_EQ(pool.size(), traits_type::in_place_delete ? 4u : 2u);
 
     pool.erase(entity[1u]);
 
     ASSERT_EQ(on_construct, 4u);
     ASSERT_EQ(on_destroy, 3u);
-    ASSERT_EQ(pool.size(), 1u);
+    ASSERT_EQ(pool.size(), traits_type::in_place_delete ? 4u : 1u);
 
     pool.erase(entity[0u]);
 
     ASSERT_EQ(on_construct, 4u);
     ASSERT_EQ(on_destroy, 4u);
-    ASSERT_EQ(pool.size(), 0u);
+    ASSERT_EQ(pool.size(), traits_type::in_place_delete ? 4u : 0u);
 
-    pool.insert(std::begin(entity), std::end(entity), 3);
+    pool.insert(entity.begin(), entity.end(), value_type{3});
 
     ASSERT_EQ(on_construct, 6u);
     ASSERT_EQ(on_destroy, 4u);
-    ASSERT_EQ(pool.size(), 2u);
+    ASSERT_EQ(pool.size(), traits_type::in_place_delete ? 6u : 2u);
 
-    ASSERT_EQ(pool.get(entity[0u]), 3);
-    ASSERT_EQ(pool.get(entity[1u]), 3);
-
-    pool.clear();
-
-    ASSERT_EQ(on_construct, 6u);
-    ASSERT_EQ(on_destroy, 6u);
-    ASSERT_EQ(pool.size(), 0u);
-}
-
-TEST(SighMixin, StableType) {
-    entt::entity entity[2u]{entt::entity{3}, entt::entity{42}}; // NOLINT
-    entt::sigh_mixin<entt::storage<test::pointer_stable>> pool;
-    entt::registry registry;
-
-    std::size_t on_construct{};
-    std::size_t on_destroy{};
-
-    pool.bind(entt::forward_as_any(registry));
-    pool.on_construct().connect<&listener<entt::registry>>(on_construct);
-    pool.on_destroy().connect<&listener<entt::registry>>(on_destroy);
-
-    ASSERT_NE(pool.push(entity[0u]), pool.entt::sparse_set::end());
-
-    pool.emplace(entity[1u]);
-
-    ASSERT_EQ(on_construct, 2u);
-    ASSERT_EQ(on_destroy, 0u);
-    ASSERT_EQ(pool.size(), 2u);
-
-    ASSERT_EQ(pool.get(entity[0u]).value, 0);
-    ASSERT_EQ(pool.get(entity[1u]).value, 0);
-
-    pool.erase(std::begin(entity), std::end(entity));
-
-    ASSERT_EQ(on_construct, 2u);
-    ASSERT_EQ(on_destroy, 2u);
-    ASSERT_EQ(pool.size(), 2u);
-
-    ASSERT_NE(pool.push(std::begin(entity), std::end(entity)), pool.entt::sparse_set::end());
-
-    ASSERT_EQ(pool.get(entity[0u]).value, 0);
-    ASSERT_EQ(pool.get(entity[1u]).value, 0);
-    ASSERT_EQ(pool.size(), 4u);
-
-    pool.erase(entity[1u]);
-
-    ASSERT_EQ(on_construct, 4u);
-    ASSERT_EQ(on_destroy, 3u);
-    ASSERT_EQ(pool.size(), 4u);
-
-    pool.erase(entity[0u]);
-
-    ASSERT_EQ(on_construct, 4u);
-    ASSERT_EQ(on_destroy, 4u);
-    ASSERT_EQ(pool.size(), 4u);
-
-    pool.insert(std::begin(entity), std::end(entity), test::pointer_stable{3});
-
-    ASSERT_EQ(on_construct, 6u);
-    ASSERT_EQ(on_destroy, 4u);
-    ASSERT_EQ(pool.size(), 6u);
-
-    ASSERT_EQ(pool.get(entity[0u]).value, 3);
-    ASSERT_EQ(pool.get(entity[1u]).value, 3);
+    ASSERT_EQ(pool.get(entity[0u]), value_type{3});
+    ASSERT_EQ(pool.get(entity[1u]), value_type{3});
 
     pool.clear();
 
@@ -193,14 +156,24 @@ TYPED_TEST(SighMixin, InsertWeakRange) {
 }
 
 TEST(SighMixin, NonDefaultConstructibleType) {
-    entt::entity entity[2u]{entt::entity{3}, entt::entity{42}}; // NOLINT
-    entt::sigh_mixin<entt::storage<test::non_default_constructible>> pool;
     entt::registry registry;
+    auto &pool = registry.storage<test::non_default_constructible>();
+    const std::array entity{registry.create(), registry.create()};
+
+    testing::StaticAssertTypeEq<decltype(pool), entt::sigh_mixin<entt::storage<test::non_default_constructible>> &>();
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
+    ASSERT_EQ(pool.size(), 0u);
+
+    pool.insert(entity.begin(), entity.begin() + 1u, 0);
+    pool.erase(entity[0u]);
+
+    ASSERT_EQ(pool.size(), 0u);
+    ASSERT_EQ(on_construct, 0u);
+    ASSERT_EQ(on_destroy, 0u);
+
     pool.on_construct().connect<&listener<entt::registry>>(on_construct);
     pool.on_destroy().connect<&listener<entt::registry>>(on_destroy);
 
@@ -221,13 +194,13 @@ TEST(SighMixin, NonDefaultConstructibleType) {
     ASSERT_EQ(on_destroy, 1u);
     ASSERT_EQ(pool.size(), 0u);
 
-    ASSERT_EQ(pool.push(std::begin(entity), std::end(entity)), pool.entt::sparse_set::end());
+    ASSERT_EQ(pool.push(entity.begin(), entity.end()), pool.entt::sparse_set::end());
 
     ASSERT_FALSE(pool.contains(entity[0u]));
     ASSERT_FALSE(pool.contains(entity[1u]));
     ASSERT_EQ(pool.size(), 0u);
 
-    pool.insert(std::begin(entity), std::end(entity), 3);
+    pool.insert(entity.begin(), entity.end(), 3);
 
     ASSERT_EQ(on_construct, 3u);
     ASSERT_EQ(on_destroy, 1u);
@@ -236,7 +209,7 @@ TEST(SighMixin, NonDefaultConstructibleType) {
     ASSERT_EQ(pool.get(entity[0u]).value, 3);
     ASSERT_EQ(pool.get(entity[1u]).value, 3);
 
-    pool.erase(std::begin(entity), std::end(entity));
+    pool.erase(entity.begin(), entity.end());
 
     ASSERT_EQ(on_construct, 3u);
     ASSERT_EQ(on_destroy, 3u);
@@ -244,30 +217,35 @@ TEST(SighMixin, NonDefaultConstructibleType) {
 }
 
 TEST(SighMixin, VoidType) {
-    entt::sigh_mixin<entt::storage<void>> pool;
     entt::registry registry;
+    auto &pool = registry.storage<void>();
+    const auto entity = registry.create();
+
+    testing::StaticAssertTypeEq<decltype(pool), entt::sigh_mixin<entt::storage<void>> &>();
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
     pool.on_construct().connect<&listener<entt::registry>>(on_construct);
     pool.on_destroy().connect<&listener<entt::registry>>(on_destroy);
 
-    pool.emplace(entt::entity{99}); // NOLINT
+    pool.emplace(entity);
 
     ASSERT_EQ(pool.type(), entt::type_id<void>());
-    ASSERT_TRUE(pool.contains(entt::entity{99}));
+    ASSERT_TRUE(pool.contains(entity));
 
     entt::sigh_mixin<entt::storage<void>> other{std::move(pool)};
 
-    ASSERT_FALSE(pool.contains(entt::entity{99})); // NOLINT
-    ASSERT_TRUE(other.contains(entt::entity{99}));
+    test::is_initialized(pool);
+
+    ASSERT_TRUE(pool.empty());
+    ASSERT_TRUE(other.contains(entity));
 
     pool = std::move(other);
+    test::is_initialized(other);
 
-    ASSERT_TRUE(pool.contains(entt::entity{99}));
-    ASSERT_FALSE(other.contains(entt::entity{99})); // NOLINT
+    ASSERT_TRUE(pool.contains(entity));
+    ASSERT_TRUE(other.empty());
 
     pool.clear();
 
@@ -278,13 +256,14 @@ TEST(SighMixin, VoidType) {
 TEST(SighMixin, StorageEntity) {
     using traits_type = entt::entt_traits<entt::entity>;
 
-    entt::sigh_mixin<entt::storage<entt::entity>> pool;
     entt::registry registry;
+    auto &pool = registry.storage<entt::entity>();
+
+    testing::StaticAssertTypeEq<decltype(pool), entt::sigh_mixin<entt::storage<entt::entity>> &>();
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
     pool.on_construct().connect<&listener<entt::registry>>(on_construct);
     pool.on_destroy().connect<&listener<entt::registry>>(on_destroy);
 
@@ -292,14 +271,14 @@ TEST(SighMixin, StorageEntity) {
 
     ASSERT_EQ(on_construct, 1u);
     ASSERT_EQ(on_destroy, 0u);
-    ASSERT_EQ(pool.size(), 2u);
+    ASSERT_EQ(pool.size(), 1u);
     ASSERT_EQ(pool.free_list(), 1u);
 
     pool.erase(entt::entity{1});
 
     ASSERT_EQ(on_construct, 1u);
     ASSERT_EQ(on_destroy, 1u);
-    ASSERT_EQ(pool.size(), 2u);
+    ASSERT_EQ(pool.size(), 1u);
     ASSERT_EQ(pool.free_list(), 0u);
 
     pool.push(traits_type::construct(0, 2));
@@ -322,11 +301,11 @@ TEST(SighMixin, StorageEntity) {
     ASSERT_EQ(on_construct, 3u);
     ASSERT_EQ(on_destroy, 3u);
 
-    pool.emplace();
-    pool.emplace(entt::entity{0});
+    pool.generate();
+    pool.generate(entt::entity{0});
 
-    entt::entity entity[1u]{};        // NOLINT
-    pool.insert(entity, entity + 1u); // NOLINT
+    std::array<entt::entity, 1u> entity{};
+    pool.generate(entity.begin(), entity.end());
 
     ASSERT_EQ(on_construct, 6u);
     ASSERT_EQ(on_destroy, 3u);
@@ -341,13 +320,14 @@ TEST(SighMixin, StorageEntity) {
 
 TYPED_TEST(SighMixin, Move) {
     using value_type = typename TestFixture::type;
+
     entt::sigh_mixin<entt::storage<value_type>> pool;
     entt::registry registry;
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
+    pool.bind(registry);
     pool.on_construct().template connect<&listener<entt::registry>>(on_construct);
     pool.on_destroy().template connect<&listener<entt::registry>>(on_destroy);
 
@@ -360,7 +340,9 @@ TYPED_TEST(SighMixin, Move) {
 
     entt::sigh_mixin<entt::storage<value_type>> other{std::move(pool)};
 
-    ASSERT_TRUE(pool.empty()); // NOLINT
+    test::is_initialized(pool);
+
+    ASSERT_TRUE(pool.empty());
     ASSERT_FALSE(other.empty());
 
     ASSERT_EQ(other.type(), entt::type_id<value_type>());
@@ -369,20 +351,22 @@ TYPED_TEST(SighMixin, Move) {
     ASSERT_EQ(other.get(entt::entity{3}), value_type{3});
 
     pool = std::move(other);
+    test::is_initialized(other);
 
     ASSERT_FALSE(pool.empty());
-    ASSERT_TRUE(other.empty()); // NOLINT
+    ASSERT_TRUE(other.empty());
 
     ASSERT_EQ(pool.index(entt::entity{3}), 0u);
     ASSERT_EQ(pool.get(entt::entity{3}), value_type{3});
 
     other = entt::sigh_mixin<entt::storage<value_type>>{};
-    other.bind(entt::forward_as_any(registry));
+    other.bind(registry);
 
-    other.emplace(entt::entity{42}, 42); // NOLINT
+    other.emplace(entt::entity{1}, 1);
     other = std::move(pool);
+    test::is_initialized(pool);
 
-    ASSERT_TRUE(pool.empty()); // NOLINT
+    ASSERT_FALSE(pool.empty());
     ASSERT_FALSE(other.empty());
 
     ASSERT_EQ(other.index(entt::entity{3}), 0u);
@@ -397,6 +381,7 @@ TYPED_TEST(SighMixin, Move) {
 TYPED_TEST(SighMixin, Swap) {
     using value_type = typename TestFixture::type;
     using traits_type = entt::component_traits<value_type>;
+
     entt::sigh_mixin<entt::storage<value_type>> pool;
     entt::sigh_mixin<entt::storage<value_type>> other;
     entt::registry registry;
@@ -404,19 +389,19 @@ TYPED_TEST(SighMixin, Swap) {
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
+    pool.bind(registry);
     pool.on_construct().template connect<&listener<entt::registry>>(on_construct);
     pool.on_destroy().template connect<&listener<entt::registry>>(on_destroy);
 
-    other.bind(entt::forward_as_any(registry));
+    other.bind(registry);
     other.on_construct().template connect<&listener<entt::registry>>(on_construct);
     other.on_destroy().template connect<&listener<entt::registry>>(on_destroy);
 
-    pool.emplace(entt::entity{42}, 41); // NOLINT
+    pool.emplace(entt::entity{4}, 1);
 
-    other.emplace(entt::entity{9}, 8); // NOLINT
-    other.emplace(entt::entity{3}, 2);
-    other.erase(entt::entity{9}); // NOLINT
+    other.emplace(entt::entity{2}, 2);
+    other.emplace(entt::entity{1}, 3);
+    other.erase(entt::entity{2});
 
     ASSERT_EQ(pool.size(), 1u);
     ASSERT_EQ(other.size(), 1u + traits_type::in_place_delete);
@@ -429,11 +414,11 @@ TYPED_TEST(SighMixin, Swap) {
     ASSERT_EQ(pool.size(), 1u + traits_type::in_place_delete);
     ASSERT_EQ(other.size(), 1u);
 
-    ASSERT_EQ(pool.index(entt::entity{3}), traits_type::in_place_delete);
-    ASSERT_EQ(other.index(entt::entity{42}), 0u);
+    ASSERT_EQ(pool.index(entt::entity{1}), traits_type::in_place_delete);
+    ASSERT_EQ(other.index(entt::entity{4}), 0u);
 
-    ASSERT_EQ(pool.get(entt::entity{3}), value_type{2});
-    ASSERT_EQ(other.get(entt::entity{42}), value_type{41});
+    ASSERT_EQ(pool.get(entt::entity{1}), value_type{3});
+    ASSERT_EQ(other.get(entt::entity{4}), value_type{1});
 
     pool.clear();
     other.clear();
@@ -442,20 +427,83 @@ TYPED_TEST(SighMixin, Swap) {
     ASSERT_EQ(on_destroy, 3u);
 }
 
+TEST(SighMixin, AutoSignal) {
+    entt::registry registry;
+    const auto entity = registry.create();
+
+    bool created{};
+    bool updated{};
+    bool destroyed{};
+
+    registry.emplace<auto_signal>(entity, created, updated, destroyed);
+    registry.replace<auto_signal>(entity, created, updated, destroyed);
+    registry.erase<auto_signal>(entity);
+
+    ASSERT_TRUE(created);
+    ASSERT_TRUE(updated);
+    ASSERT_TRUE(destroyed);
+
+    ASSERT_TRUE(registry.storage<auto_signal>().empty());
+    ASSERT_TRUE(registry.valid(entity));
+
+    created = updated = destroyed = false;
+
+    registry.emplace<auto_signal>(entity, created, updated, destroyed);
+    registry.replace<auto_signal>(entity, created, updated, destroyed);
+    registry.destroy(entity);
+
+    ASSERT_TRUE(created);
+    ASSERT_TRUE(updated);
+    ASSERT_TRUE(destroyed);
+
+    ASSERT_TRUE(registry.storage<auto_signal>().empty());
+    ASSERT_FALSE(registry.valid(entity));
+}
+
+TYPED_TEST(SighMixin, Registry) {
+    using value_type = typename TestFixture::type;
+
+    entt::registry registry;
+    entt::sigh_mixin<entt::storage<value_type>> pool;
+
+    ASSERT_FALSE(pool);
+
+    pool.bind(registry);
+
+    ASSERT_TRUE(pool);
+    ASSERT_EQ(&pool.registry(), &registry);
+    ASSERT_EQ(&std::as_const(pool).registry(), &registry);
+}
+
+ENTT_DEBUG_TYPED_TEST(SighMixinDeathTest, Registry) {
+    using value_type = typename TestFixture::type;
+    entt::sigh_mixin<entt::storage<value_type>> pool;
+    ASSERT_DEATH([[maybe_unused]] auto &registry = pool.registry(), "");
+    ASSERT_DEATH([[maybe_unused]] const auto &registry = std::as_const(pool).registry(), "");
+}
+
 TYPED_TEST(SighMixin, CustomRegistry) {
     using value_type = typename TestFixture::type;
-    entt::basic_sigh_mixin<entt::basic_storage<value_type, test::custom_entity>, custom_registry> pool;
-    custom_registry registry;
+    using registry_type = test::custom_registry<test::entity>;
+
+    registry_type registry;
+    entt::basic_sigh_mixin<entt::basic_storage<value_type, test::entity>, registry_type> pool;
+    const std::array entity{registry.create(), registry.create()};
+
+    ASSERT_FALSE(pool);
+
+    pool.bind(registry);
+
+    ASSERT_TRUE(pool);
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(static_cast<entt::basic_registry<test::custom_entity> &>(registry)));
-    pool.on_construct().template connect<&listener<custom_registry>>(on_construct);
-    pool.on_destroy().template connect<&listener<custom_registry>>(on_destroy);
+    pool.on_construct().template connect<&listener<registry_type>>(on_construct);
+    pool.on_destroy().template connect<&listener<registry_type>>(on_destroy);
 
-    pool.emplace(test::custom_entity{3});
-    pool.emplace(test::custom_entity{42}); // NOLINT
+    pool.emplace(entity[0u]);
+    pool.emplace(entity[1u]);
 
     ASSERT_EQ(on_construct, 2u);
     ASSERT_EQ(on_destroy, 0u);
@@ -466,18 +514,27 @@ TYPED_TEST(SighMixin, CustomRegistry) {
     ASSERT_EQ(on_destroy, 2u);
 }
 
+ENTT_DEBUG_TYPED_TEST(SighMixinDeathTest, CustomRegistry) {
+    using value_type = typename TestFixture::type;
+    using registry_type = test::custom_registry<test::entity>;
+    entt::basic_sigh_mixin<entt::basic_storage<value_type, test::entity>, registry_type> pool;
+    ASSERT_DEATH([[maybe_unused]] auto &registry = pool.registry(), "");
+    ASSERT_DEATH([[maybe_unused]] const auto &registry = std::as_const(pool).registry(), "");
+}
+
 TYPED_TEST(SighMixin, CustomAllocator) {
     using value_type = typename TestFixture::type;
-    const test::throwing_allocator<entt::entity> allocator{};
-    entt::sigh_mixin<entt::basic_storage<value_type, entt::entity, test::throwing_allocator<value_type>>> pool{allocator};
+    using storage_type = entt::sigh_mixin<entt::basic_storage<value_type, entt::entity, test::throwing_allocator<value_type>>>;
+    using registry_type = typename storage_type::registry_type;
 
-    using registry_type = typename decltype(pool)::registry_type;
+    const test::throwing_allocator<entt::entity> allocator{};
+    storage_type pool{allocator};
     registry_type registry;
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
+    pool.bind(registry);
     pool.on_construct().template connect<&listener<registry_type>>(on_construct);
     pool.on_destroy().template connect<&listener<registry_type>>(on_destroy);
 
@@ -490,26 +547,28 @@ TYPED_TEST(SighMixin, CustomAllocator) {
 
     decltype(pool) other{std::move(pool), allocator};
 
-    ASSERT_TRUE(pool.empty()); // NOLINT
+    test::is_initialized(pool);
+
+    ASSERT_TRUE(pool.empty());
     ASSERT_FALSE(other.empty());
-    ASSERT_EQ(pool.capacity(), 0u); // NOLINT
     ASSERT_NE(other.capacity(), 0u);
     ASSERT_EQ(other.size(), 2u);
 
     pool = std::move(other);
+    test::is_initialized(other);
 
     ASSERT_FALSE(pool.empty());
-    ASSERT_TRUE(other.empty());      // NOLINT
-    ASSERT_EQ(other.capacity(), 0u); // NOLINT
+    ASSERT_TRUE(other.empty());
     ASSERT_NE(pool.capacity(), 0u);
     ASSERT_EQ(pool.size(), 2u);
 
+    other = {};
     pool.swap(other);
     pool = std::move(other);
+    test::is_initialized(other);
 
     ASSERT_FALSE(pool.empty());
-    ASSERT_TRUE(other.empty());      // NOLINT
-    ASSERT_EQ(other.capacity(), 0u); // NOLINT
+    ASSERT_TRUE(other.empty());
     ASSERT_NE(pool.capacity(), 0u);
     ASSERT_EQ(pool.size(), 2u);
 
@@ -524,19 +583,20 @@ TYPED_TEST(SighMixin, CustomAllocator) {
 
 TYPED_TEST(SighMixin, ThrowingAllocator) {
     using value_type = typename TestFixture::type;
-    entt::sigh_mixin<entt::basic_storage<value_type, entt::entity, test::throwing_allocator<value_type>>> pool{};
-    typename std::decay_t<decltype(pool)>::base_type &base = pool;
+    using storage_type = entt::sigh_mixin<entt::basic_storage<value_type, entt::entity, test::throwing_allocator<value_type>>>;
+    using registry_type = typename storage_type::registry_type;
 
-    using registry_type = typename decltype(pool)::registry_type;
+    storage_type pool{};
+    typename storage_type::base_type &base = pool;
     registry_type registry;
 
-    constexpr auto packed_page_size = entt::component_traits<typename decltype(pool)::value_type>::page_size;
-    constexpr auto sparse_page_size = entt::entt_traits<typename decltype(pool)::entity_type>::page_size;
+    constexpr auto packed_page_size = entt::component_traits<value_type>::page_size;
+    constexpr auto sparse_page_size = entt::entt_traits<entt::entity>::page_size;
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
+    pool.bind(registry);
     pool.on_construct().template connect<&listener<registry_type>>(on_construct);
     pool.on_destroy().template connect<&listener<registry_type>>(on_destroy);
 
@@ -574,19 +634,19 @@ TYPED_TEST(SighMixin, ThrowingAllocator) {
     ASSERT_TRUE(pool.empty());
 
     pool.emplace(entt::entity{0}, 0);
-    const entt::entity entity[2u]{entt::entity{1}, entt::entity{sparse_page_size}}; // NOLINT
+    const std::array entity{entt::entity{1}, entt::entity{sparse_page_size}};
     pool.get_allocator().template throw_counter<entt::entity>(1u);
 
-    ASSERT_THROW(pool.insert(std::begin(entity), std::end(entity), value_type{0}), test::throwing_allocator_exception);
+    ASSERT_THROW(pool.insert(entity.begin(), entity.end(), value_type{0}), test::throwing_allocator_exception);
     ASSERT_TRUE(pool.contains(entt::entity{1}));
     ASSERT_FALSE(pool.contains(entt::entity{sparse_page_size}));
 
     pool.erase(entt::entity{1});
-    const value_type components[2u]{value_type{1}, value_type{sparse_page_size}}; // NOLINT
+    const std::array component{value_type{1}, value_type{sparse_page_size}};
     pool.get_allocator().template throw_counter<entt::entity>(0u);
     pool.compact();
 
-    ASSERT_THROW(pool.insert(std::begin(entity), std::end(entity), std::begin(components)), test::throwing_allocator_exception);
+    ASSERT_THROW(pool.insert(entity.begin(), entity.end(), component.begin()), test::throwing_allocator_exception);
     ASSERT_TRUE(pool.contains(entt::entity{1}));
     ASSERT_FALSE(pool.contains(entt::entity{sparse_page_size}));
 
@@ -595,36 +655,38 @@ TYPED_TEST(SighMixin, ThrowingAllocator) {
 }
 
 TEST(SighMixin, ThrowingComponent) {
-    entt::sigh_mixin<entt::storage<test::throwing_type>> pool;
-    using registry_type = typename decltype(pool)::registry_type;
+    using storage_type = entt::sigh_mixin<entt::storage<test::throwing_type>>;
+    using registry_type = typename storage_type::registry_type;
+
+    storage_type pool;
     registry_type registry;
 
     std::size_t on_construct{};
     std::size_t on_destroy{};
 
-    pool.bind(entt::forward_as_any(registry));
+    pool.bind(registry);
     pool.on_construct().connect<&listener<registry_type>>(on_construct);
     pool.on_destroy().connect<&listener<registry_type>>(on_destroy);
 
-    const entt::entity entity[2u]{entt::entity{42}, entt::entity{1}}; // NOLINT
-    const test::throwing_type value[2u]{true, false};                 // NOLINT
+    const std::array entity{entt::entity{3}, entt::entity{1}};
+    const std::array<test::throwing_type, 2u> value{true, false};
 
     // strong exception safety
     ASSERT_THROW(pool.emplace(entity[0u], value[0u]), test::throwing_type_exception);
     ASSERT_TRUE(pool.empty());
 
     // basic exception safety
-    ASSERT_THROW(pool.insert(std::begin(entity), std::end(entity), value[0u]), test::throwing_type_exception);
+    ASSERT_THROW(pool.insert(entity.begin(), entity.end(), value[0u]), test::throwing_type_exception);
     ASSERT_EQ(pool.size(), 0u);
     ASSERT_FALSE(pool.contains(entity[1u]));
 
     // basic exception safety
-    ASSERT_THROW(pool.insert(std::begin(entity), std::end(entity), std::begin(value)), test::throwing_type_exception);
+    ASSERT_THROW(pool.insert(entity.begin(), entity.end(), value.begin()), test::throwing_type_exception);
     ASSERT_EQ(pool.size(), 0u);
     ASSERT_FALSE(pool.contains(entity[1u]));
 
     // basic exception safety
-    ASSERT_THROW(pool.insert(std::rbegin(entity), std::rend(entity), std::rbegin(value)), test::throwing_type_exception);
+    ASSERT_THROW(pool.insert(entity.rbegin(), entity.rend(), value.rbegin()), test::throwing_type_exception);
     ASSERT_EQ(pool.size(), 1u);
     ASSERT_TRUE(pool.contains(entity[1u]));
     ASSERT_EQ(pool.get(entity[1u]), value[1u]);
