@@ -1,6 +1,7 @@
 #include <algorithm>
 #include <array>
 #include <cstddef>
+#include <cstdint>
 #include <memory>
 #include <utility>
 #include <gtest/gtest.h>
@@ -65,7 +66,7 @@ struct fat: empty {
     std::array<double, 4u> value{};
 };
 
-enum class enum_class : unsigned short int {
+enum class enum_class : std::uint8_t {
     foo = 0u,
     bar = 1u
 };
@@ -89,16 +90,16 @@ struct MetaAny: ::testing::Test {
     void SetUp() override {
         using namespace entt::literals;
 
-        entt::meta<empty>()
+        entt::meta_factory<empty>{}
             .type("empty"_hs)
             .dtor<empty::destroy>();
 
-        entt::meta<fat>()
+        entt::meta_factory<fat>{}
             .type("fat"_hs)
             .base<empty>()
             .dtor<fat::destroy>();
 
-        entt::meta<clazz>()
+        entt::meta_factory<clazz>{}
             .type("clazz"_hs)
             .data<&clazz::value>("value"_hs)
             .func<&clazz::member>("member"_hs)
@@ -116,38 +117,13 @@ struct MetaAny: ::testing::Test {
 
 using MetaAnyDeathTest = MetaAny;
 
-TEST_F(MetaAny, SBO) {
-    entt::meta_any any{'c'};
-
-    ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::meta_any_policy::owner);
-    ASSERT_FALSE(any.try_cast<std::size_t>());
-    ASSERT_EQ(any.cast<char>(), 'c');
-    ASSERT_NE(any.data(), nullptr);
-    ASSERT_EQ(any, entt::meta_any{'c'});
-    ASSERT_NE(entt::meta_any{'h'}, any);
-}
-
-TEST_F(MetaAny, NoSBO) {
-    const fat instance{.1, .2, .3, .4};
-    entt::meta_any any{instance};
-
-    ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::meta_any_policy::owner);
-    ASSERT_FALSE(any.try_cast<std::size_t>());
-    ASSERT_EQ(any.cast<fat>(), instance);
-    ASSERT_NE(any.data(), nullptr);
-    ASSERT_EQ(any, entt::meta_any{instance});
-    ASSERT_NE(any, fat{});
-}
-
 TEST_F(MetaAny, Empty) {
     entt::meta_any any{};
 
     ASSERT_FALSE(any);
     ASSERT_FALSE(any.type());
     ASSERT_FALSE(any.try_cast<std::size_t>());
-    ASSERT_EQ(any.data(), nullptr);
+    ASSERT_EQ(any.base().data(), nullptr);
     ASSERT_EQ(any, entt::meta_any{});
     ASSERT_NE(entt::meta_any{'c'}, any);
 
@@ -160,13 +136,89 @@ TEST_F(MetaAny, Empty) {
     ASSERT_FALSE(std::as_const(any).as_associative_container());
 }
 
+TEST_F(MetaAny, Context) {
+    entt::meta_any any{};
+    const entt::meta_ctx ctx{};
+
+    ASSERT_EQ(&any.context(), &entt::locator<entt::meta_ctx>::value_or());
+    ASSERT_NE(&any.context(), &ctx);
+
+    any = entt::meta_any{entt::meta_ctx_arg, ctx};
+
+    ASSERT_NE(&any.context(), &entt::locator<entt::meta_ctx>::value_or());
+    ASSERT_EQ(&any.context(), &ctx);
+}
+
+TEST_F(MetaAny, SBO) {
+    entt::meta_any any{'c'};
+
+    ASSERT_TRUE(any);
+    ASSERT_TRUE(any.base().owner());
+    ASSERT_EQ(any.policy(), entt::meta_any_policy::embedded);
+    ASSERT_FALSE(any.try_cast<std::size_t>());
+    ASSERT_EQ(any.cast<char>(), 'c');
+    ASSERT_NE(any.data(), nullptr);
+    ASSERT_EQ(any, entt::meta_any{'c'});
+    ASSERT_NE(entt::meta_any{'h'}, any);
+}
+
+TEST_F(MetaAny, NoSBO) {
+    const fat instance{.1, .2, .3, .4};
+    entt::meta_any any{instance};
+
+    ASSERT_TRUE(any);
+    ASSERT_TRUE(any.base().owner());
+    ASSERT_EQ(any.policy(), entt::meta_any_policy::dynamic);
+    ASSERT_FALSE(any.try_cast<std::size_t>());
+    ASSERT_EQ(any.cast<fat>(), instance);
+    ASSERT_NE(any.data(), nullptr);
+    ASSERT_EQ(any, entt::meta_any{instance});
+    ASSERT_NE(any, fat{});
+}
+
+TEST_F(MetaAny, SBOInPlaceConstruction) {
+    std::unique_ptr<int> elem = std::make_unique<int>(2);
+    // NOLINTNEXTLINE(clang-analyzer-cplusplus.NewDeleteLeaks)
+    entt::meta_any any{std::in_place, elem.release()};
+
+    ASSERT_TRUE(any);
+    ASSERT_TRUE(any.base().owner());
+    ASSERT_EQ(any.base().policy(), entt::any_policy::dynamic);
+    ASSERT_FALSE(any.try_cast<std::size_t>());
+    ASSERT_EQ(any.cast<int>(), 2);
+    ASSERT_NE(any.base().data(), nullptr);
+    ASSERT_EQ(any, entt::meta_any{2});
+    ASSERT_NE(entt::meta_any{0}, any);
+
+    auto other = any.as_ref();
+
+    ASSERT_TRUE(other);
+    ASSERT_FALSE(other.base().owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::ref);
+    ASSERT_FALSE(other.try_cast<std::size_t>());
+    ASSERT_EQ(other.cast<int>(), 2);
+    ASSERT_NE(other.base().data(), nullptr);
+    ASSERT_EQ(other, entt::meta_any{2});
+    ASSERT_NE(entt::meta_any{0}, other);
+}
+
+TEST_F(MetaAny, SBOInPlaceNullptrConstruction) {
+    int *instance = nullptr;
+    entt::meta_any any{std::in_place, instance};
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.base().owner());
+    ASSERT_FALSE(any.try_cast<int>());
+    ASSERT_EQ(any.base().data(), nullptr);
+}
+
 TEST_F(MetaAny, SBOInPlaceTypeConstruction) {
     entt::meta_any any{std::in_place_type<int>, 3};
 
     ASSERT_TRUE(any);
     ASSERT_FALSE(any.try_cast<std::size_t>());
     ASSERT_EQ(any.cast<int>(), 3);
-    ASSERT_NE(any.data(), nullptr);
+    ASSERT_NE(any.base().data(), nullptr);
     ASSERT_EQ(any, (entt::meta_any{std::in_place_type<int>, 3}));
     ASSERT_EQ(any, entt::meta_any{3});
     ASSERT_NE(entt::meta_any{1}, any);
@@ -178,15 +230,15 @@ TEST_F(MetaAny, SBOAsRefConstruction) {
     auto any = entt::forward_as_meta(value);
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::meta_any_policy::ref);
+    ASSERT_FALSE(any.base().owner());
+    ASSERT_EQ(any.base().policy(), entt::any_policy::ref);
     ASSERT_EQ(any.type(), entt::resolve<int>());
 
     ASSERT_FALSE(any.try_cast<std::size_t>());
     ASSERT_EQ(any.cast<int &>(), 1);
     ASSERT_EQ(any.cast<const int &>(), 1);
     ASSERT_EQ(any.cast<int>(), 1);
-    ASSERT_EQ(any.data(), &value);
-    ASSERT_EQ(std::as_const(any).data(), &value);
+    ASSERT_EQ(any.base().data(), &value);
 
     ASSERT_EQ(any, entt::forward_as_meta(value));
     ASSERT_NE(any, entt::forward_as_meta(compare));
@@ -198,14 +250,14 @@ TEST_F(MetaAny, SBOAsRefConstruction) {
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
-    ASSERT_EQ(std::as_const(any).data(), &value);
+    ASSERT_EQ(any.base().data(), &value);
 
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
     ASSERT_EQ(any.type(), entt::resolve<int>());
     ASSERT_EQ(any.cast<int>(), 1);
-    ASSERT_EQ(other.data(), any.data());
+    ASSERT_EQ(other.base().data(), any.base().data());
 }
 
 TEST_F(MetaAny, SBOAsConstRefConstruction) {
@@ -214,14 +266,14 @@ TEST_F(MetaAny, SBOAsConstRefConstruction) {
     auto any = entt::forward_as_meta(value);
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::meta_any_policy::cref);
+    ASSERT_FALSE(any.base().owner());
+    ASSERT_EQ(any.base().policy(), entt::any_policy::cref);
     ASSERT_EQ(any.type(), entt::resolve<int>());
 
     ASSERT_FALSE(any.try_cast<std::size_t>());
     ASSERT_EQ(any.cast<const int &>(), 1);
     ASSERT_EQ(any.cast<int>(), 1);
-    ASSERT_EQ(any.data(), nullptr);
-    ASSERT_EQ(std::as_const(any).data(), &value);
+    ASSERT_EQ(any.base().data(), &value);
 
     ASSERT_EQ(any, entt::forward_as_meta(value));
     ASSERT_NE(any, entt::forward_as_meta(compare));
@@ -233,14 +285,14 @@ TEST_F(MetaAny, SBOAsConstRefConstruction) {
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
-    ASSERT_EQ(std::as_const(any).data(), &value);
+    ASSERT_EQ(any.base().data(), &value);
 
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
     ASSERT_EQ(any.type(), entt::resolve<int>());
     ASSERT_EQ(any.cast<int>(), 1);
-    ASSERT_EQ(other.data(), any.data());
+    ASSERT_EQ(other.base().data(), any.base().data());
 }
 
 ENTT_DEBUG_TEST_F(MetaAnyDeathTest, SBOAsConstRefConstruction) {
@@ -319,11 +371,15 @@ TEST_F(MetaAny, SBOMoveAssignment) {
     ASSERT_NE(other, entt::meta_any{0});
 }
 
-ENTT_DEBUG_TEST_F(MetaAnyDeathTest, SBOSelfMoveAssignment) {
+TEST_F(MetaAnyDeathTest, SBOSelfMoveAssignment) {
     entt::meta_any any{3};
 
     // avoid warnings due to self-assignment
-    ASSERT_DEATH(any = std::move(*&any), "");
+    any = std::move(*&any);
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.type());
+    ASSERT_EQ(any.base().data(), nullptr);
 }
 
 TEST_F(MetaAny, SBODirectAssignment) {
@@ -451,6 +507,43 @@ TEST_F(MetaAny, SBOAsConstRefTransferValue) {
     ASSERT_EQ(value, 3);
 }
 
+TEST_F(MetaAny, NoSBOInPlaceConstruction) {
+    const fat instance{.1, .2, .3, .4};
+    std::unique_ptr<fat> elem = std::make_unique<fat>(instance);
+    // NOLINTNEXTLINE(clang-analyzer-cplusplus.NewDeleteLeaks)
+    entt::meta_any any{std::in_place, elem.release()};
+
+    ASSERT_TRUE(any);
+    ASSERT_TRUE(any.base().owner());
+    ASSERT_EQ(any.base().policy(), entt::any_policy::dynamic);
+    ASSERT_FALSE(any.try_cast<std::size_t>());
+    ASSERT_EQ(any.cast<fat>(), instance);
+    ASSERT_NE(any.base().data(), nullptr);
+    ASSERT_EQ(any, entt::meta_any{instance});
+    ASSERT_NE(any, fat{});
+
+    auto other = any.as_ref();
+
+    ASSERT_TRUE(other);
+    ASSERT_FALSE(other.base().owner());
+    ASSERT_EQ(other.policy(), entt::any_policy::ref);
+    ASSERT_FALSE(other.try_cast<std::size_t>());
+    ASSERT_EQ(other.cast<fat>(), instance);
+    ASSERT_NE(other.base().data(), nullptr);
+    ASSERT_EQ(any, entt::meta_any{instance});
+    ASSERT_NE(any, fat{});
+}
+
+TEST_F(MetaAny, NoSBOInPlaceNullptrConstruction) {
+    fat *instance = nullptr;
+    entt::meta_any any{std::in_place, instance};
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.base().owner());
+    ASSERT_FALSE(any.try_cast<fat>());
+    ASSERT_EQ(any.base().data(), nullptr);
+}
+
 TEST_F(MetaAny, NoSBOInPlaceTypeConstruction) {
     const fat instance{.1, .2, .3, .4};
     entt::meta_any any{std::in_place_type<fat>, instance};
@@ -458,7 +551,7 @@ TEST_F(MetaAny, NoSBOInPlaceTypeConstruction) {
     ASSERT_TRUE(any);
     ASSERT_FALSE(any.try_cast<std::size_t>());
     ASSERT_EQ(any.cast<fat>(), instance);
-    ASSERT_NE(any.data(), nullptr);
+    ASSERT_NE(any.base().data(), nullptr);
     ASSERT_EQ(any, (entt::meta_any{std::in_place_type<fat>, instance}));
     ASSERT_EQ(any, entt::meta_any{instance});
     ASSERT_NE(entt::meta_any{fat{}}, any);
@@ -469,15 +562,15 @@ TEST_F(MetaAny, NoSBOAsRefConstruction) {
     auto any = entt::forward_as_meta(instance);
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::meta_any_policy::ref);
+    ASSERT_FALSE(any.base().owner());
+    ASSERT_EQ(any.base().policy(), entt::any_policy::ref);
     ASSERT_EQ(any.type(), entt::resolve<fat>());
 
     ASSERT_FALSE(any.try_cast<std::size_t>());
     ASSERT_EQ(any.cast<fat &>(), instance);
     ASSERT_EQ(any.cast<const fat &>(), instance);
     ASSERT_EQ(any.cast<fat>(), instance);
-    ASSERT_EQ(any.data(), &instance);
-    ASSERT_EQ(std::as_const(any).data(), &instance);
+    ASSERT_EQ(any.base().data(), &instance);
 
     ASSERT_EQ(any, entt::forward_as_meta(instance));
 
@@ -488,14 +581,14 @@ TEST_F(MetaAny, NoSBOAsRefConstruction) {
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<fat>());
-    ASSERT_EQ(std::as_const(any).data(), &instance);
+    ASSERT_EQ(any.base().data(), &instance);
 
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
     ASSERT_EQ(any.type(), entt::resolve<fat>());
     ASSERT_EQ(any, entt::meta_any{instance});
-    ASSERT_EQ(other.data(), any.data());
+    ASSERT_EQ(other.base().data(), any.base().data());
 }
 
 TEST_F(MetaAny, NoSBOAsConstRefConstruction) {
@@ -503,14 +596,14 @@ TEST_F(MetaAny, NoSBOAsConstRefConstruction) {
     auto any = entt::forward_as_meta(instance);
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::meta_any_policy::cref);
+    ASSERT_FALSE(any.base().owner());
+    ASSERT_EQ(any.base().policy(), entt::any_policy::cref);
     ASSERT_EQ(any.type(), entt::resolve<fat>());
 
     ASSERT_FALSE(any.try_cast<std::size_t>());
     ASSERT_EQ(any.cast<const fat &>(), instance);
     ASSERT_EQ(any.cast<fat>(), instance);
-    ASSERT_EQ(any.data(), nullptr);
-    ASSERT_EQ(std::as_const(any).data(), &instance);
+    ASSERT_EQ(any.base().data(), &instance);
 
     ASSERT_EQ(any, entt::forward_as_meta(instance));
 
@@ -521,14 +614,14 @@ TEST_F(MetaAny, NoSBOAsConstRefConstruction) {
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<fat>());
-    ASSERT_EQ(std::as_const(any).data(), &instance);
+    ASSERT_EQ(any.base().data(), &instance);
 
     auto other = any.as_ref();
 
     ASSERT_TRUE(other);
     ASSERT_EQ(any.type(), entt::resolve<fat>());
     ASSERT_EQ(any, entt::meta_any{instance});
-    ASSERT_EQ(other.data(), any.data());
+    ASSERT_EQ(other.base().data(), any.base().data());
 }
 
 ENTT_DEBUG_TEST_F(MetaAnyDeathTest, NoSBOAsConstRefConstruction) {
@@ -612,12 +705,16 @@ TEST_F(MetaAny, NoSBOMoveAssignment) {
     ASSERT_NE(other, fat{});
 }
 
-ENTT_DEBUG_TEST_F(MetaAnyDeathTest, NoSBOSelfMoveAssignment) {
+TEST_F(MetaAnyDeathTest, NoSBOSelfMoveAssignment) {
     const fat instance{.1, .2, .3, .4};
     entt::meta_any any{instance};
 
     // avoid warnings due to self-assignment
-    ASSERT_DEATH(any = std::move(*&any), "");
+    any = std::move(*&any);
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.type());
+    ASSERT_EQ(any.base().data(), nullptr);
 }
 
 TEST_F(MetaAny, NoSBODirectAssignment) {
@@ -636,7 +733,7 @@ TEST_F(MetaAny, NoSBOAssignValue) {
     const entt::meta_any other{fat{.0, .1, .2, .3}};
     const entt::meta_any invalid{'c'};
 
-    const void *addr = std::as_const(any).data();
+    const void *addr = any.base().data();
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.cast<const fat &>(), (fat{.1, .2, .3, .4}));
@@ -644,7 +741,7 @@ TEST_F(MetaAny, NoSBOAssignValue) {
     ASSERT_TRUE(any.assign(other));
     ASSERT_FALSE(any.assign(invalid));
     ASSERT_EQ(any.cast<const fat &>(), (fat{.0, .1, .2, .3}));
-    ASSERT_EQ(addr, std::as_const(any).data());
+    ASSERT_EQ(addr, any.base().data());
 }
 
 TEST_F(MetaAny, NoSBOConvertAssignValue) {
@@ -652,12 +749,12 @@ TEST_F(MetaAny, NoSBOConvertAssignValue) {
     const entt::meta_any other{fat{.0, .1, .2, .3}};
     const entt::meta_any invalid{'c'};
 
-    const void *addr = std::as_const(any).data();
+    const void *addr = any.base().data();
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(any.assign(other));
     ASSERT_FALSE(any.assign(invalid));
-    ASSERT_EQ(addr, std::as_const(any).data());
+    ASSERT_EQ(addr, any.base().data());
 }
 
 TEST_F(MetaAny, NoSBOAsRefAssignValue) {
@@ -693,7 +790,7 @@ TEST_F(MetaAny, NoSBOAsConstRefAssignValue) {
 TEST_F(MetaAny, NoSBOTransferValue) {
     entt::meta_any any{fat{.1, .2, .3, .4}};
 
-    const void *addr = std::as_const(any).data();
+    const void *addr = any.base().data();
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.cast<const fat &>(), (fat{.1, .2, .3, .4}));
@@ -701,39 +798,39 @@ TEST_F(MetaAny, NoSBOTransferValue) {
     ASSERT_TRUE(any.assign(fat{.0, .1, .2, .3}));
     ASSERT_FALSE(any.assign('c'));
     ASSERT_EQ(any.cast<const fat &>(), (fat{.0, .1, .2, .3}));
-    ASSERT_EQ(addr, std::as_const(any).data());
+    ASSERT_EQ(addr, any.base().data());
 }
 
 TEST_F(MetaAny, NoSBOTransferConstValue) {
     const fat instance{.0, .1, .2, .3};
     entt::meta_any any{fat{.1, .2, .3, .4}};
 
-    const void *addr = std::as_const(any).data();
+    const void *addr = any.base().data();
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.cast<const fat &>(), (fat{.1, .2, .3, .4}));
 
     ASSERT_TRUE(any.assign(entt::forward_as_meta(instance)));
     ASSERT_EQ(any.cast<const fat &>(), (fat{.0, .1, .2, .3}));
-    ASSERT_EQ(addr, std::as_const(any).data());
+    ASSERT_EQ(addr, any.base().data());
 }
 
 TEST_F(MetaAny, NoSBOConvertTransferValue) {
     entt::meta_any any{empty{}};
 
-    const void *addr = std::as_const(any).data();
+    const void *addr = any.base().data();
 
     ASSERT_TRUE(any);
     ASSERT_TRUE(any.assign(fat{.0, .1, .2, .3}));
     ASSERT_FALSE(any.assign('c'));
-    ASSERT_EQ(addr, std::as_const(any).data());
+    ASSERT_EQ(addr, any.base().data());
 }
 
 TEST_F(MetaAny, NoSBOAsRefTransferValue) {
     fat instance{.1, .2, .3, .4};
     entt::meta_any any{entt::forward_as_meta(instance)};
 
-    const void *addr = std::as_const(any).data();
+    const void *addr = any.base().data();
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.cast<const fat &>(), (fat{.1, .2, .3, .4}));
@@ -742,14 +839,14 @@ TEST_F(MetaAny, NoSBOAsRefTransferValue) {
     ASSERT_FALSE(any.assign('c'));
     ASSERT_EQ(any.cast<const fat &>(), (fat{.0, .1, .2, .3}));
     ASSERT_EQ(instance, (fat{.0, .1, .2, .3}));
-    ASSERT_EQ(addr, std::as_const(any).data());
+    ASSERT_EQ(addr, any.base().data());
 }
 
 TEST_F(MetaAny, NoSBOAsConstRefTransferValue) {
     const fat instance{.1, .2, .3, .4};
     entt::meta_any any{entt::forward_as_meta(instance)};
 
-    const void *addr = std::as_const(any).data();
+    const void *addr = any.base().data();
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.cast<const fat &>(), (fat{.1, .2, .3, .4}));
@@ -758,21 +855,51 @@ TEST_F(MetaAny, NoSBOAsConstRefTransferValue) {
     ASSERT_FALSE(any.assign('c'));
     ASSERT_EQ(any.cast<const fat &>(), (fat{.1, .2, .3, .4}));
     ASSERT_EQ(instance, (fat{.1, .2, .3, .4}));
-    ASSERT_EQ(addr, std::as_const(any).data());
+    ASSERT_EQ(addr, any.base().data());
 }
 
 TEST_F(MetaAny, VoidInPlaceTypeConstruction) {
     entt::meta_any any{std::in_place_type<void>};
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.policy(), entt::meta_any_policy::owner);
+    ASSERT_FALSE(any.base().owner());
+    ASSERT_EQ(any.base().policy(), entt::any_policy::empty);
     ASSERT_FALSE(any.try_cast<char>());
-    ASSERT_EQ(any.data(), nullptr);
+    ASSERT_EQ(any.base().data(), nullptr);
     ASSERT_EQ(any.type(), entt::resolve<void>());
     ASSERT_EQ(any, entt::meta_any{std::in_place_type<void>});
     ASSERT_NE(entt::meta_any{3}, any);
 }
 
+TEST_F(MetaAny, VoidAsRefConstruction) {
+    entt::meta_any any{std::in_place_type<void>};
+
+    ASSERT_TRUE(any);
+    ASSERT_FALSE(any.base().owner());
+    ASSERT_EQ(any.base().policy(), entt::any_policy::empty);
+    ASSERT_EQ(any.type(), entt::resolve<void>());
+
+    ASSERT_FALSE(any.try_cast<std::size_t>());
+    ASSERT_EQ(any.base().data(), nullptr);
+
+    ASSERT_EQ(any, entt::meta_any{std::in_place_type<void>});
+    ASSERT_EQ(entt::meta_any{std::in_place_type<void>}, any);
+    ASSERT_NE(entt::meta_any{3}, any);
+
+    any = entt::meta_any{std::in_place_type<void>};
+
+    ASSERT_TRUE(any);
+    ASSERT_EQ(any.type(), entt::resolve<void>());
+    ASSERT_EQ(any.base().data(), nullptr);
+
+    auto other = any.as_ref();
+
+    ASSERT_TRUE(other);
+    ASSERT_EQ(any.type(), entt::resolve<void>());
+    ASSERT_EQ(any.base().data(), nullptr);
+    ASSERT_EQ(other.base().data(), any.base().data());
+}
+
 TEST_F(MetaAny, VoidCopyConstruction) {
     const entt::meta_any any{std::in_place_type<void>};
     entt::meta_any other{any};
@@ -834,11 +961,15 @@ TEST_F(MetaAny, VoidMoveAssignment) {
     ASSERT_EQ(other, entt::meta_any{std::in_place_type<void>});
 }
 
-ENTT_DEBUG_TEST_F(MetaAnyDeathTest, VoidSelfMoveAssignment) {
+TEST_F(MetaAnyDeathTest, VoidSelfMoveAssignment) {
     entt::meta_any any{std::in_place_type<void>};
 
     // avoid warnings due to self-assignment
-    ASSERT_DEATH(any = std::move(*&any), "");
+    any = std::move(*&any);
+
+    ASSERT_FALSE(any);
+    ASSERT_FALSE(any.type());
+    ASSERT_EQ(any.base().data(), nullptr);
 }
 
 TEST_F(MetaAny, SBOMoveInvalidate) {
@@ -919,7 +1050,7 @@ TEST_F(MetaAny, Emplace) {
     ASSERT_TRUE(any);
     ASSERT_FALSE(any.try_cast<std::size_t>());
     ASSERT_EQ(any.cast<int>(), 3);
-    ASSERT_NE(any.data(), nullptr);
+    ASSERT_NE(any.base().data(), nullptr);
     ASSERT_EQ(any, (entt::meta_any{std::in_place_type<int>, 3}));
     ASSERT_EQ(any, entt::meta_any{3});
     ASSERT_NE(entt::meta_any{1}, any);
@@ -930,7 +1061,7 @@ TEST_F(MetaAny, EmplaceVoid) {
     any.emplace<void>();
 
     ASSERT_TRUE(any);
-    ASSERT_EQ(any.data(), nullptr);
+    ASSERT_EQ(any.base().data(), nullptr);
     ASSERT_EQ(any.type(), entt::resolve<void>());
     ASSERT_EQ(any, (entt::meta_any{std::in_place_type<void>}));
 }
@@ -972,11 +1103,11 @@ TEST_F(MetaAny, NoSBOSwap) {
 TEST_F(MetaAny, VoidSwap) {
     entt::meta_any lhs{std::in_place_type<void>};
     entt::meta_any rhs{std::in_place_type<void>};
-    const auto *pre = lhs.data();
+    const auto *pre = lhs.base().data();
 
     std::swap(lhs, rhs);
 
-    ASSERT_EQ(pre, lhs.data());
+    ASSERT_EQ(pre, lhs.base().data());
 }
 
 TEST_F(MetaAny, SBOWithNoSBOSwap) {
@@ -1051,13 +1182,13 @@ TEST_F(MetaAny, AsRef) {
     auto ref = any.as_ref();
     auto cref = std::as_const(any).as_ref();
 
-    ASSERT_EQ(any.try_cast<int>(), any.data());
-    ASSERT_EQ(ref.try_cast<int>(), any.data());
+    ASSERT_EQ(any.try_cast<int>(), any.base().data());
+    ASSERT_EQ(ref.try_cast<int>(), any.base().data());
     ASSERT_EQ(cref.try_cast<int>(), nullptr);
 
-    ASSERT_EQ(any.try_cast<const int>(), any.data());
-    ASSERT_EQ(ref.try_cast<const int>(), any.data());
-    ASSERT_EQ(cref.try_cast<const int>(), any.data());
+    ASSERT_EQ(any.try_cast<const int>(), any.base().data());
+    ASSERT_EQ(ref.try_cast<const int>(), any.base().data());
+    ASSERT_EQ(cref.try_cast<const int>(), any.base().data());
 
     ASSERT_EQ(any.cast<int>(), 3);
     ASSERT_EQ(ref.cast<int>(), 3);
@@ -1082,15 +1213,15 @@ TEST_F(MetaAny, AsRef) {
     std::swap(ref, cref);
 
     ASSERT_EQ(ref.try_cast<int>(), nullptr);
-    ASSERT_EQ(cref.try_cast<int>(), any.data());
+    ASSERT_EQ(cref.try_cast<int>(), any.base().data());
 
     ref = ref.as_ref();
     cref = std::as_const(cref).as_ref();
 
     ASSERT_EQ(ref.try_cast<int>(), nullptr);
     ASSERT_EQ(cref.try_cast<int>(), nullptr);
-    ASSERT_EQ(ref.try_cast<const int>(), any.data());
-    ASSERT_EQ(cref.try_cast<const int>(), any.data());
+    ASSERT_EQ(ref.try_cast<const int>(), any.base().data());
+    ASSERT_EQ(cref.try_cast<const int>(), any.base().data());
 
     ASSERT_EQ(ref.cast<const int &>(), 1);
     ASSERT_EQ(cref.cast<const int &>(), 1);
@@ -1104,16 +1235,16 @@ TEST_F(MetaAny, AsRef) {
     ASSERT_EQ(cref.cast<int &>(), 3);
     ASSERT_EQ(ref.cast<const int &>(), 3);
     ASSERT_EQ(cref.cast<const int &>(), 3);
-    ASSERT_NE(ref.try_cast<int>(), any.data());
-    ASSERT_NE(cref.try_cast<int>(), any.data());
+    ASSERT_NE(ref.try_cast<int>(), any.base().data());
+    ASSERT_NE(cref.try_cast<int>(), any.base().data());
 
     any.emplace<void>();
     ref = any.as_ref();
     cref = std::as_const(any).as_ref();
 
     ASSERT_TRUE(any);
-    ASSERT_FALSE(ref);
-    ASSERT_FALSE(cref);
+    ASSERT_TRUE(ref);
+    ASSERT_TRUE(cref);
 }
 
 ENTT_DEBUG_TEST_F(MetaAnyDeathTest, AsRef) {
@@ -1173,9 +1304,9 @@ TEST_F(MetaAny, TryCast) {
     ASSERT_EQ(any.type(), entt::resolve<fat>());
     ASSERT_EQ(any.try_cast<void>(), nullptr);
     ASSERT_NE(any.try_cast<empty>(), nullptr);
-    ASSERT_EQ(any.try_cast<fat>(), any.data());
+    ASSERT_EQ(any.try_cast<fat>(), any.base().data());
     ASSERT_EQ(std::as_const(any).try_cast<empty>(), any.try_cast<empty>());
-    ASSERT_EQ(std::as_const(any).try_cast<fat>(), any.data());
+    ASSERT_EQ(std::as_const(any).try_cast<fat>(), any.base().data());
     ASSERT_EQ(std::as_const(any).try_cast<int>(), nullptr);
 }
 
@@ -1392,7 +1523,7 @@ TEST_F(MetaAny, UnmanageableType) {
     ASSERT_TRUE(other);
 
     ASSERT_EQ(any.type(), entt::resolve<unmanageable>());
-    ASSERT_NE(any.data(), nullptr);
+    ASSERT_NE(any.base().data(), nullptr);
     ASSERT_EQ(any.try_cast<int>(), nullptr);
     ASSERT_NE(any.try_cast<unmanageable>(), nullptr);
 
@@ -1460,6 +1591,6 @@ TEST_F(MetaAny, ForwardAsMeta) {
     ASSERT_EQ(ref.cast<const int &>(), 3);
     ASSERT_EQ(cref.cast<const int &>(), 3);
 
-    ASSERT_NE(any.data(), &value);
-    ASSERT_EQ(ref.data(), &value);
+    ASSERT_NE(any.base().data(), &value);
+    ASSERT_EQ(ref.base().data(), &value);
 }
