@@ -87,47 +87,39 @@ struct array {
     int local[4];                // NOLINT
 };
 
-enum class property_type : entt::id_type {
-    random,
-    value
-};
-
 struct MetaData: ::testing::Test {
     void SetUp() override {
         using namespace entt::literals;
 
-        entt::meta<base>()
+        entt::meta_factory<base>{}
             .type("base"_hs)
             .dtor<base::destroy>()
             .data<&base::value>("value"_hs);
 
-        entt::meta<derived>()
+        entt::meta_factory<derived>{}
             .type("derived"_hs)
             .base<base>()
             .dtor<derived::destroy>()
             .data<&base::value>("value_from_base"_hs);
 
-        entt::meta<clazz>()
+        entt::meta_factory<clazz>{}
             .type("clazz"_hs)
             .data<&clazz::i, entt::as_ref_t>("i"_hs)
             .custom<char>('c')
             .traits(test::meta_traits::one | test::meta_traits::two | test::meta_traits::three)
-            .prop(3u, 0)
             .data<&clazz::i, entt::as_cref_t>("ci"_hs)
             .data<&clazz::j>("j"_hs)
             .traits(test::meta_traits::one)
-            .prop("true"_hs, 1)
             .data<&clazz::h>("h"_hs)
             .traits(test::meta_traits::two)
-            .prop(static_cast<entt::id_type>(property_type::random), 2)
             .data<&clazz::k>("k"_hs)
             .traits(test::meta_traits::three)
-            .prop(static_cast<entt::id_type>(property_type::value), 3)
+            .data<'c'>("l"_hs)
             .data<&clazz::instance>("base"_hs)
             .data<&clazz::i, entt::as_void_t>("void"_hs)
             .conv<int>();
 
-        entt::meta<setter_getter>()
+        entt::meta_factory<setter_getter>{}
             .type("setter_getter"_hs)
             .data<&setter_getter::static_setter, &setter_getter::static_getter>("x"_hs)
             .data<&setter_getter::setter, &setter_getter::getter>("y"_hs)
@@ -136,11 +128,11 @@ struct MetaData: ::testing::Test {
             .data<nullptr, &setter_getter::getter>("z_ro"_hs)
             .data<nullptr, &setter_getter::value>("value"_hs);
 
-        entt::meta<multi_setter>()
+        entt::meta_factory<multi_setter>{}
             .type("multi_setter"_hs)
             .data<entt::value_list<&multi_setter::from_double, &multi_setter::from_string>, &multi_setter::value>("value"_hs);
 
-        entt::meta<array>()
+        entt::meta_factory<array>{}
             .type("array"_hs)
             .data<&array::global>("global"_hs)
             .data<&array::local>("local"_hs);
@@ -155,40 +147,20 @@ struct MetaData: ::testing::Test {
 
 using MetaDataDeathTest = MetaData;
 
-TEST_F(MetaData, Functionalities) {
-    using namespace entt::literals;
+TEST_F(MetaData, SafeWhenEmpty) {
+    const entt::meta_data data{};
 
-    auto data = entt::resolve<clazz>().data("i"_hs);
-    clazz instance{};
-
-    ASSERT_TRUE(data);
-
-    ASSERT_EQ(data, data);
-    ASSERT_NE(data, entt::meta_data{});
-    ASSERT_FALSE(data != data);
-    ASSERT_TRUE(data == data);
-
-    ASSERT_EQ(data.arity(), 1u);
-    ASSERT_EQ(data.type(), entt::resolve<int>());
-    ASSERT_EQ(data.arg(0u), entt::resolve<int>());
+    ASSERT_FALSE(data);
+    ASSERT_EQ(data, entt::meta_data{});
+    ASSERT_EQ(data.arity(), 0u);
     ASSERT_FALSE(data.is_const());
     ASSERT_FALSE(data.is_static());
-    ASSERT_EQ(data.get(instance).cast<int>(), 0);
-    ASSERT_TRUE(data.set(instance, 1));
-    ASSERT_EQ(data.get(instance).cast<int>(), 1);
-
-    for(auto curr: data.prop()) {
-        ASSERT_EQ(curr.first, 3u);
-        ASSERT_EQ(curr.second.value(), 0);
-    }
-
-    ASSERT_FALSE(data.prop(2));
-    ASSERT_FALSE(data.prop('c'));
-
-    auto prop = data.prop(3u);
-
-    ASSERT_TRUE(prop);
-    ASSERT_EQ(prop.value(), 0);
+    ASSERT_EQ(data.type(), entt::meta_type{});
+    ASSERT_FALSE(data.set({}, 0));
+    ASSERT_FALSE(data.get({}));
+    ASSERT_EQ(data.arg(0u), entt::meta_type{});
+    ASSERT_EQ(data.traits<test::meta_traits>(), test::meta_traits::none);
+    ASSERT_EQ(static_cast<const char *>(data.custom()), nullptr);
 }
 
 TEST_F(MetaData, UserTraits) {
@@ -208,7 +180,7 @@ ENTT_DEBUG_TEST_F(MetaDataDeathTest, UserTraits) {
 
     using traits_type = entt::internal::meta_traits;
     constexpr auto value = traits_type{static_cast<std::underlying_type_t<traits_type>>(traits_type::_user_defined_traits) + 1u};
-    ASSERT_DEATH(entt::meta<clazz>().data<&clazz::i>("j"_hs).traits(value), "");
+    ASSERT_DEATH(entt::meta_factory<clazz>{}.data<&clazz::i>("j"_hs).traits(value), "");
 }
 
 TEST_F(MetaData, Custom) {
@@ -224,8 +196,38 @@ TEST_F(MetaData, Custom) {
 ENTT_DEBUG_TEST_F(MetaDataDeathTest, Custom) {
     using namespace entt::literals;
 
-    ASSERT_DEATH([[maybe_unused]] int value = entt::resolve<clazz>().data("i"_hs).custom(), "");
-    ASSERT_DEATH([[maybe_unused]] char value = entt::resolve<clazz>().data("j"_hs).custom(), "");
+    ASSERT_DEATH([[maybe_unused]] const int value = entt::resolve<clazz>().data("i"_hs).custom(), "");
+    ASSERT_DEATH([[maybe_unused]] const char value = entt::resolve<clazz>().data("j"_hs).custom(), "");
+}
+
+TEST_F(MetaData, Comparison) {
+    using namespace entt::literals;
+
+    auto data = entt::resolve<clazz>().data("i"_hs);
+
+    ASSERT_TRUE(data);
+
+    ASSERT_EQ(data, data);
+    ASSERT_NE(data, entt::meta_data{});
+    ASSERT_FALSE(data != data);
+    ASSERT_TRUE(data == data);
+}
+
+TEST_F(MetaData, NonConst) {
+    using namespace entt::literals;
+
+    auto data = entt::resolve<clazz>().data("i"_hs);
+    clazz instance{};
+
+    ASSERT_TRUE(data);
+    ASSERT_EQ(data.arity(), 1u);
+    ASSERT_EQ(data.type(), entt::resolve<int>());
+    ASSERT_EQ(data.arg(0u), entt::resolve<int>());
+    ASSERT_FALSE(data.is_const());
+    ASSERT_FALSE(data.is_static());
+    ASSERT_EQ(data.get(instance).cast<int>(), 0);
+    ASSERT_TRUE(data.set(instance, 1));
+    ASSERT_EQ(data.get(instance).cast<int>(), 1);
 }
 
 TEST_F(MetaData, Const) {
@@ -243,19 +245,6 @@ TEST_F(MetaData, Const) {
     ASSERT_EQ(data.get(instance).cast<int>(), 1);
     ASSERT_FALSE(data.set(instance, 1));
     ASSERT_EQ(data.get(instance).cast<int>(), 1);
-
-    for(auto curr: data.prop()) {
-        ASSERT_EQ(curr.first, "true"_hs);
-        ASSERT_EQ(curr.second.value(), 1);
-    }
-
-    ASSERT_FALSE(data.prop(false));
-    ASSERT_FALSE(data.prop('c'));
-
-    auto prop = data.prop("true"_hs);
-
-    ASSERT_TRUE(prop);
-    ASSERT_EQ(prop.value(), 1);
 }
 
 TEST_F(MetaData, Static) {
@@ -272,19 +261,6 @@ TEST_F(MetaData, Static) {
     ASSERT_EQ(data.get({}).cast<int>(), 2);
     ASSERT_TRUE(data.set({}, 1));
     ASSERT_EQ(data.get({}).cast<int>(), 1);
-
-    for(auto curr: data.prop()) {
-        ASSERT_EQ(curr.first, static_cast<entt::id_type>(property_type::random));
-        ASSERT_EQ(curr.second.value(), 2);
-    }
-
-    ASSERT_FALSE(data.prop(static_cast<entt::id_type>(property_type::value)));
-    ASSERT_FALSE(data.prop('c'));
-
-    auto prop = data.prop(static_cast<entt::id_type>(property_type::random));
-
-    ASSERT_TRUE(prop);
-    ASSERT_EQ(prop.value(), 2);
 }
 
 TEST_F(MetaData, ConstStatic) {
@@ -301,19 +277,22 @@ TEST_F(MetaData, ConstStatic) {
     ASSERT_EQ(data.get({}).cast<int>(), 3);
     ASSERT_FALSE(data.set({}, 1));
     ASSERT_EQ(data.get({}).cast<int>(), 3);
+}
 
-    for(auto curr: data.prop()) {
-        ASSERT_EQ(curr.first, static_cast<entt::id_type>(property_type::value));
-        ASSERT_EQ(curr.second.value(), 3);
-    }
+TEST_F(MetaData, Literal) {
+    using namespace entt::literals;
 
-    ASSERT_FALSE(data.prop(static_cast<entt::id_type>(property_type::random)));
-    ASSERT_FALSE(data.prop('c'));
+    auto data = entt::resolve<clazz>().data("l"_hs);
 
-    auto prop = data.prop(static_cast<entt::id_type>(property_type::value));
-
-    ASSERT_TRUE(prop);
-    ASSERT_EQ(prop.value(), 3);
+    ASSERT_TRUE(data);
+    ASSERT_EQ(data.arity(), 1u);
+    ASSERT_EQ(data.type(), entt::resolve<char>());
+    ASSERT_EQ(data.arg(0u), entt::resolve<char>());
+    ASSERT_TRUE(data.is_const());
+    ASSERT_TRUE(data.is_static());
+    ASSERT_EQ(data.get({}).cast<char>(), 'c');
+    ASSERT_FALSE(data.set({}, 'a'));
+    ASSERT_EQ(data.get({}).cast<char>(), 'c');
 }
 
 TEST_F(MetaData, GetMetaAnyArg) {
@@ -622,7 +601,6 @@ TEST_F(MetaData, AsRef) {
     ASSERT_EQ(data.arity(), 1u);
     ASSERT_EQ(data.type(), entt::resolve<int>());
     ASSERT_EQ(data.arg(0u), entt::resolve<int>());
-    ASSERT_NE(data.prop().cbegin(), data.prop().cend());
     ASSERT_EQ(instance.i, 0);
 
     data.get(instance).cast<int &>() = 3;
@@ -642,7 +620,6 @@ TEST_F(MetaData, AsConstRef) {
     ASSERT_EQ(data.arg(0u), entt::resolve<int>());
     ASSERT_EQ(data.get(instance).cast<const int &>(), 0);
     ASSERT_EQ(data.get(instance).cast<int>(), 0);
-    ASSERT_EQ(data.prop().cbegin(), data.prop().cend());
     ASSERT_EQ(instance.i, 0);
 }
 
@@ -696,14 +673,14 @@ TEST_F(MetaData, ReRegistration) {
     ASSERT_EQ(node.details->data.size(), 1u);
     ASSERT_TRUE(type.data("value"_hs));
 
-    entt::meta<base>().data<&base::value>("field"_hs);
+    entt::meta_factory<base>{}.data<&base::value>("field"_hs);
 
     ASSERT_TRUE(node.details);
     ASSERT_EQ(node.details->data.size(), 2u);
     ASSERT_TRUE(type.data("value"_hs));
     ASSERT_TRUE(type.data("field"_hs));
 
-    entt::meta<base>()
+    entt::meta_factory<base>{}
         .data<&base::value>("field"_hs)
         .traits(test::meta_traits::one)
         .custom<int>(3)
@@ -721,8 +698,8 @@ TEST_F(MetaData, CollisionAndReuse) {
     ASSERT_FALSE(entt::resolve<clazz>().data("cj"_hs));
     ASSERT_TRUE(entt::resolve<clazz>().data("j"_hs).is_const());
 
-    ASSERT_NO_THROW(entt::meta<clazz>().data<&clazz::i>("j"_hs));
-    ASSERT_NO_THROW(entt::meta<clazz>().data<&clazz::j>("cj"_hs));
+    ASSERT_NO_THROW(entt::meta_factory<clazz>{}.data<&clazz::i>("j"_hs));
+    ASSERT_NO_THROW(entt::meta_factory<clazz>{}.data<&clazz::j>("cj"_hs));
 
     ASSERT_TRUE(entt::resolve<clazz>().data("j"_hs));
     ASSERT_TRUE(entt::resolve<clazz>().data("cj"_hs));
