@@ -93,24 +93,24 @@ struct MetaFunc: ::testing::Test {
     void SetUp() override {
         using namespace entt::literals;
 
-        entt::meta<double>()
+        entt::meta_factory<double>{}
             .type("double"_hs)
             .func<&double_member>("member"_hs);
 
-        entt::meta<base>()
+        entt::meta_factory<base>{}
             .type("base"_hs)
             .func<&base::setter>("setter"_hs)
             .func<fake_member>("fake_member"_hs)
             .func<fake_const_member>("fake_const_member"_hs);
 
-        entt::meta<derived>()
+        entt::meta_factory<derived>{}
             .type("derived"_hs)
             .base<base>()
             .func<&base::setter>("setter_from_base"_hs)
             .func<&base::getter>("getter_from_base"_hs)
             .func<&base::static_setter>("static_setter_from_base"_hs);
 
-        entt::meta<function>()
+        entt::meta_factory<function>{}
             .type("func"_hs)
             .func<&entt::registry::emplace_or_replace<function>, entt::as_ref_t>("emplace"_hs)
             .traits(test::meta_traits::one | test::meta_traits::two | test::meta_traits::three)
@@ -119,17 +119,12 @@ struct MetaFunc: ::testing::Test {
             .func<entt::overload<int(int, int)>(&function::f)>("f2"_hs)
             .traits(test::meta_traits::two)
             .custom<int>(2)
-            .prop("true"_hs, false)
             .func<entt::overload<int(int) const>(&function::f)>("f1"_hs)
             .traits(test::meta_traits::one)
-            .prop("true"_hs, false)
             .func<&function::g>("g"_hs)
             .custom<char>('c')
-            .prop("true"_hs, false)
             .func<function::h>("h"_hs)
-            .prop("true"_hs, false)
             .func<function::k>("k"_hs)
-            .prop("true"_hs, false)
             .func<&function::v, entt::as_void_t>("v"_hs)
             .func<&function::a, entt::as_ref_t>("a"_hs)
             .func<&function::a, entt::as_cref_t>("ca"_hs)
@@ -143,7 +138,7 @@ struct MetaFunc: ::testing::Test {
     std::size_t reset_and_check() {
         std::size_t count = 0;
 
-        for(auto func: entt::resolve<function>().func()) {
+        for(const auto &func: entt::resolve<function>().func()) {
             for(auto curr = func.second; curr; curr = curr.next()) {
                 ++count;
             }
@@ -151,7 +146,7 @@ struct MetaFunc: ::testing::Test {
 
         SetUp();
 
-        for(auto func: entt::resolve<function>().func()) {
+        for(const auto &func: entt::resolve<function>().func()) {
             for(auto curr = func.second; curr; curr = curr.next()) {
                 --count;
             }
@@ -163,48 +158,25 @@ struct MetaFunc: ::testing::Test {
 
 using MetaFuncDeathTest = MetaFunc;
 
-TEST_F(MetaFunc, Functionalities) {
-    using namespace entt::literals;
+TEST_F(MetaFunc, SafeWhenEmpty) {
+    const entt::meta_func func{};
+    entt::meta_any *args = nullptr;
 
-    auto func = entt::resolve<function>().func("f2"_hs);
-    function instance{};
-
-    ASSERT_TRUE(func);
-
-    ASSERT_EQ(func, func);
-    ASSERT_NE(func, entt::meta_func{});
-    ASSERT_FALSE(func != func);
-    ASSERT_TRUE(func == func);
-
-    ASSERT_EQ(func.arity(), 2u);
+    ASSERT_FALSE(func);
+    ASSERT_EQ(func, entt::meta_func{});
+    ASSERT_EQ(func.arity(), 0u);
     ASSERT_FALSE(func.is_const());
     ASSERT_FALSE(func.is_static());
-    ASSERT_EQ(func.ret(), entt::resolve<int>());
-    ASSERT_EQ(func.arg(0u), entt::resolve<int>());
-    ASSERT_EQ(func.arg(1u), entt::resolve<int>());
-    ASSERT_FALSE(func.arg(2u));
-
-    auto any = func.invoke(instance, 3, 2);
-    auto empty = func.invoke(instance);
-
-    ASSERT_FALSE(empty);
-    ASSERT_TRUE(any);
-    ASSERT_EQ(any.type(), entt::resolve<int>());
-    ASSERT_EQ(any.cast<int>(), 4);
-    ASSERT_EQ(instance.value, 3);
-
-    for(auto curr: func.prop()) {
-        ASSERT_EQ(curr.first, "true"_hs);
-        ASSERT_FALSE(curr.second.value().template cast<bool>());
-    }
-
-    ASSERT_FALSE(func.prop(false));
-    ASSERT_FALSE(func.prop('c'));
-
-    auto prop = func.prop("true"_hs);
-
-    ASSERT_TRUE(prop);
-    ASSERT_FALSE(prop.value().cast<bool>());
+    ASSERT_EQ(func.ret(), entt::meta_type{});
+    ASSERT_EQ(func.arg(0u), entt::meta_type{});
+    ASSERT_EQ(func.arg(1u), entt::meta_type{});
+    ASSERT_FALSE(func.invoke({}, args, 0u));
+    ASSERT_FALSE(func.invoke({}, args, 1u));
+    ASSERT_FALSE(func.invoke({}));
+    ASSERT_FALSE(func.invoke({}, 'c'));
+    ASSERT_EQ(func.traits<test::meta_traits>(), test::meta_traits::none);
+    ASSERT_EQ(static_cast<const char *>(func.custom()), nullptr);
+    ASSERT_EQ(func.next(), func);
 }
 
 TEST_F(MetaFunc, UserTraits) {
@@ -224,7 +196,7 @@ ENTT_DEBUG_TEST_F(MetaFuncDeathTest, UserTraits) {
 
     using traits_type = entt::internal::meta_traits;
     constexpr auto value = traits_type{static_cast<std::underlying_type_t<traits_type>>(traits_type::_user_defined_traits) + 1u};
-    ASSERT_DEATH(entt::meta<function>().func<&function::g>("g"_hs).traits(value), "");
+    ASSERT_DEATH(entt::meta_factory<function>{}.func<&function::g>("g"_hs).traits(value), "");
 }
 
 TEST_F(MetaFunc, Custom) {
@@ -240,8 +212,45 @@ TEST_F(MetaFunc, Custom) {
 ENTT_DEBUG_TEST_F(MetaFuncDeathTest, Custom) {
     using namespace entt::literals;
 
-    ASSERT_DEATH([[maybe_unused]] int value = entt::resolve<function>().func("g"_hs).custom(), "");
-    ASSERT_DEATH([[maybe_unused]] char value = entt::resolve<function>().func("h"_hs).custom(), "");
+    ASSERT_DEATH([[maybe_unused]] const int value = entt::resolve<function>().func("g"_hs).custom(), "");
+    ASSERT_DEATH([[maybe_unused]] const char value = entt::resolve<function>().func("h"_hs).custom(), "");
+}
+
+TEST_F(MetaFunc, Comparison) {
+    using namespace entt::literals;
+
+    auto func = entt::resolve<function>().func("f2"_hs);
+
+    ASSERT_TRUE(func);
+    ASSERT_EQ(func, func);
+    ASSERT_NE(func, entt::meta_func{});
+    ASSERT_FALSE(func != func);
+    ASSERT_TRUE(func == func);
+}
+
+TEST_F(MetaFunc, NonConst) {
+    using namespace entt::literals;
+
+    auto func = entt::resolve<function>().func("f2"_hs);
+    function instance{};
+
+    ASSERT_TRUE(func);
+    ASSERT_EQ(func.arity(), 2u);
+    ASSERT_FALSE(func.is_const());
+    ASSERT_FALSE(func.is_static());
+    ASSERT_EQ(func.ret(), entt::resolve<int>());
+    ASSERT_EQ(func.arg(0u), entt::resolve<int>());
+    ASSERT_EQ(func.arg(1u), entt::resolve<int>());
+    ASSERT_FALSE(func.arg(2u));
+
+    auto any = func.invoke(instance, 3, 2);
+    auto empty = func.invoke(instance);
+
+    ASSERT_FALSE(empty);
+    ASSERT_TRUE(any);
+    ASSERT_EQ(any.type(), entt::resolve<int>());
+    ASSERT_EQ(any.cast<int>(), 4);
+    ASSERT_EQ(instance.value, 3);
 }
 
 TEST_F(MetaFunc, Const) {
@@ -265,19 +274,6 @@ TEST_F(MetaFunc, Const) {
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
     ASSERT_EQ(any.cast<int>(), 8);
-
-    for(auto curr: func.prop()) {
-        ASSERT_EQ(curr.first, "true"_hs);
-        ASSERT_FALSE(curr.second.value().template cast<bool>());
-    }
-
-    ASSERT_FALSE(func.prop(false));
-    ASSERT_FALSE(func.prop('c'));
-
-    auto prop = func.prop("true"_hs);
-
-    ASSERT_TRUE(prop);
-    ASSERT_FALSE(prop.value().cast<bool>());
 }
 
 TEST_F(MetaFunc, RetVoid) {
@@ -299,19 +295,6 @@ TEST_F(MetaFunc, RetVoid) {
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<void>());
     ASSERT_EQ(instance.value, 16);
-
-    for(auto curr: func.prop()) {
-        ASSERT_EQ(curr.first, "true"_hs);
-        ASSERT_FALSE(curr.second.value().template cast<bool>());
-    }
-
-    ASSERT_FALSE(func.prop(false));
-    ASSERT_FALSE(func.prop('c'));
-
-    auto prop = func.prop("true"_hs);
-
-    ASSERT_TRUE(prop);
-    ASSERT_FALSE(prop.value().cast<bool>());
 }
 
 TEST_F(MetaFunc, Static) {
@@ -336,19 +319,6 @@ TEST_F(MetaFunc, Static) {
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
     ASSERT_EQ(any.cast<int>(), 6);
-
-    for(auto curr: func.prop()) {
-        ASSERT_EQ(curr.first, "true"_hs);
-        ASSERT_FALSE(curr.second.value().template cast<bool>());
-    }
-
-    ASSERT_FALSE(func.prop(false));
-    ASSERT_FALSE(func.prop('c'));
-
-    auto prop = func.prop("true"_hs);
-
-    ASSERT_TRUE(prop);
-    ASSERT_FALSE(prop.value().cast<bool>());
 }
 
 TEST_F(MetaFunc, StaticRetVoid) {
@@ -371,19 +341,6 @@ TEST_F(MetaFunc, StaticRetVoid) {
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<void>());
     ASSERT_EQ(instance.value, 3);
-
-    for(auto curr: func.prop()) {
-        ASSERT_EQ(curr.first, "true"_hs);
-        ASSERT_FALSE(curr.second.value().template cast<bool>());
-    }
-
-    ASSERT_FALSE(func.prop(false));
-    ASSERT_FALSE(func.prop('c'));
-
-    auto prop = func.prop("true"_hs);
-
-    ASSERT_TRUE(prop);
-    ASSERT_FALSE(prop.value().cast<bool>());
 }
 
 TEST_F(MetaFunc, StaticAsMember) {
@@ -401,8 +358,6 @@ TEST_F(MetaFunc, StaticAsMember) {
     ASSERT_EQ(func.arg(0u), entt::resolve<int>());
     ASSERT_FALSE(func.arg(1u));
 
-    ASSERT_EQ(func.prop().cbegin(), func.prop().cend());
-
     ASSERT_FALSE(func.invoke({}, 3));
     ASSERT_FALSE(func.invoke(std::as_const(instance), 3));
 
@@ -425,8 +380,6 @@ TEST_F(MetaFunc, StaticAsConstMember) {
     ASSERT_EQ(func.ret(), entt::resolve<int>());
     ASSERT_FALSE(func.arg(0u));
 
-    ASSERT_EQ(func.prop().cbegin(), func.prop().cend());
-
     ASSERT_FALSE(func.invoke({}));
     ASSERT_TRUE(func.invoke(instance));
 
@@ -449,8 +402,6 @@ TEST_F(MetaFunc, NonClassTypeMember) {
     ASSERT_EQ(func.ret(), entt::resolve<double>());
     ASSERT_FALSE(func.arg(0u));
 
-    ASSERT_EQ(func.prop().cbegin(), func.prop().cend());
-
     ASSERT_FALSE(func.invoke({}));
     ASSERT_TRUE(func.invoke(instance));
 
@@ -661,7 +612,7 @@ TEST_F(MetaFunc, Overloaded) {
 
     ASSERT_FALSE(type.func("f2"_hs).next());
 
-    entt::meta<function>()
+    entt::meta_factory<function>{}
         // this should not overwrite traits and custom data
         .func<entt::overload<int(int, int)>(&function::f)>("f2"_hs)
         // this should put traits and custom data on the new overload instead
@@ -685,7 +636,7 @@ TEST_F(MetaFunc, Overloaded) {
 TEST_F(MetaFunc, OverloadedOrder) {
     using namespace entt::literals;
 
-    entt::meta<function>()
+    entt::meta_factory<function>{}
         .func<entt::overload<int(int, int)>(&function::f)>("f2"_hs)
         .func<entt::overload<int(int) const>(&function::f)>("f2"_hs);
 
@@ -725,7 +676,7 @@ TEST_F(MetaFunc, ReRegistration) {
     ASSERT_TRUE(type.invoke("f1"_hs, instance, 0));
     ASSERT_FALSE(type.invoke("f1"_hs, instance, 0, 0));
 
-    entt::meta<function>()
+    entt::meta_factory<function>{}
         .func<entt::overload<int(int, int)>(&function::f)>("f"_hs)
         .func<entt::overload<int(int) const>(&function::f)>("f"_hs);
 
@@ -736,5 +687,15 @@ TEST_F(MetaFunc, ReRegistration) {
     ASSERT_TRUE(type.invoke("f"_hs, instance, 0));
     ASSERT_TRUE(type.invoke("f"_hs, instance, 0, 0));
 
+    entt::meta_factory<function>{}
+        .func<entt::overload<int(int, int)>(&function::f)>("f"_hs)
+        .traits(test::meta_traits::one)
+        .custom<int>(3)
+        // this should not overwrite traits and custom data
+        .func<entt::overload<int(int, int)>(&function::f)>("f"_hs);
+
+    ASSERT_EQ(type.func("f"_hs).traits<test::meta_traits>(), test::meta_traits::one);
+    ASSERT_NE(static_cast<const int *>(type.func("f"_hs).custom()), nullptr);
+
     ASSERT_EQ(reset_and_check(), 0u);
 }
